@@ -29,8 +29,11 @@
 
 #include <kio/copyjob.h>
 #include <kio/job.h>
+#include <kio/mkpathjob.h>
 #include <kio/deletejob.h>
 #include <kio/paste.h>
+#include <kio/pastejob.h>
+#include <kio/restorejob.h>
 #include <kioglobal_p.h>
 #include <kprotocolinfo.h>
 #include <kurlmimedata.h>
@@ -59,7 +62,7 @@ using namespace KIO;
 
 static QString homeTmpDir()
 {
-    return QStandardPaths::writableLocation(QStandardPaths::DataLocation) + '/';
+    return QStandardPaths::writableLocation(QStandardPaths::DataLocation) + QDir::separator();
 }
 static QString destDir()
 {
@@ -147,7 +150,7 @@ static void checkTestDirectory(const QString &path)
 static void createTestDirectory(const QString &path)
 {
     QDir dir;
-    bool ok = dir.mkdir(path);
+    bool ok = dir.mkpath(path);
     if (!ok) {
         qFatal("couldn't create %s", qPrintable(path));
     }
@@ -170,11 +173,11 @@ public:
     {
         setShowProgressInfo(false);
     }
-    virtual void jobError(KIO::Job *job)
+    void jobError(KIO::Job *job) Q_DECL_OVERRIDE
     {
         qFatal("%s", qPrintable(job->errorString()));
     }
-    virtual bool copiedFileWasModified(const QUrl &src, const QUrl &dest, const QDateTime &srcTime, const QDateTime &destTime)
+    bool copiedFileWasModified(const QUrl &src, const QUrl &dest, const QDateTime &srcTime, const QDateTime &destTime) Q_DECL_OVERRIDE
     {
         Q_UNUSED(src);
         m_dest = dest;
@@ -182,7 +185,7 @@ public:
         Q_UNUSED(destTime);
         return true;
     }
-    virtual bool confirmDeletion(const QList<QUrl> &files)
+    bool confirmDeletion(const QList<QUrl> &files) Q_DECL_OVERRIDE
     {
         m_files = files;
         return m_nextReplyToConfirmDeletion;
@@ -214,18 +217,17 @@ void FileUndoManagerTest::initTestCase()
 {
     qDebug("initTestCase");
 
-    // TODO: needs QStandardPaths::isTestModeEnabled() in ksycoca.cpp when launching kbuildsycoca
-    //QStandardPaths::enableTestMode(true);
+    QStandardPaths::enableTestMode(true);
 
     // Get kio_trash to share our environment so that it writes trashrc to the right kdehome
     qputenv("KDE_FORK_SLAVES", "yes");
+    qputenv("KIOSLAVE_ENABLE_TESTMODE", "1");
 
     // Start with a clean base dir
     cleanupTestCase();
 
-    QDir dir; // TT: why not a static method?
     if (!QFile::exists(homeTmpDir())) {
-        bool ok = dir.mkdir(homeTmpDir());
+        bool ok = QDir().mkpath(homeTmpDir());
         if (!ok) {
             qFatal("Couldn't create %s", qPrintable(homeTmpDir()));
         }
@@ -237,7 +239,7 @@ void FileUndoManagerTest::initTestCase()
 #endif
     createTestDirectory(srcSubDir());
 
-    QDir().mkdir(destDir());
+    QDir().mkpath(destDir());
     QVERIFY(QFileInfo(destDir()).isDir());
 
     QVERIFY(!FileUndoManager::self()->undoAvailable());
@@ -481,9 +483,34 @@ void FileUndoManagerTest::testCreateDir()
     QVERIFY(!QFile::exists(path));
 }
 
+void FileUndoManagerTest::testMkpath()
+{
+    const QString parent = srcSubDir() + "mkpath";
+    const QString path = parent + "/subdir";
+    QVERIFY(!QFile::exists(path));
+    const QUrl url = QUrl::fromLocalFile(path);
+
+    KIO::Job *job = KIO::mkpath(url);
+    job->setUiDelegate(0);
+    FileUndoManager::self()->recordJob(FileUndoManager::Mkpath, QList<QUrl>(), url, job);
+    QVERIFY(job->exec());
+    QVERIFY(QFileInfo(path).isDir());
+
+    m_uiInterface->clear();
+    m_uiInterface->setNextReplyToConfirmDeletion(true);
+    doUndo();
+
+    QVERIFY(!FileUndoManager::self()->undoAvailable());
+    QCOMPARE(m_uiInterface->files().count(), 2);   // confirmDeletion was called
+    QCOMPARE(m_uiInterface->files()[0].toLocalFile(), path);
+    QCOMPARE(m_uiInterface->files()[1].toLocalFile(), parent);
+
+    QVERIFY(!QFile::exists(path));
+}
+
 void FileUndoManagerTest::testTrashFiles()
 {
-    if (!KProtocolInfo::isKnownProtocol("trash")) {
+    if (!KProtocolInfo::isKnownProtocol(QStringLiteral("trash"))) {
         QSKIP("kio_trash not installed");
     }
 
@@ -492,7 +519,7 @@ void FileUndoManagerTest::testTrashFiles()
     lst.append(QUrl::fromLocalFile(srcSubDir()));
     KIO::Job *job = KIO::trash(lst, KIO::HideProgressInfo);
     job->setUiDelegate(0);
-    FileUndoManager::self()->recordJob(FileUndoManager::Trash, lst, QUrl("trash:/"), job);
+    FileUndoManager::self()->recordJob(FileUndoManager::Trash, lst, QUrl(QStringLiteral("trash:/")), job);
 
     bool ok = job->exec();
     QVERIFY(ok);
@@ -506,7 +533,7 @@ void FileUndoManagerTest::testTrashFiles()
 
     // check trash?
     // Let's just check that it's not empty. kio_trash has its own unit tests anyway.
-    KConfig cfg("trashrc", KConfig::SimpleConfig);
+    KConfig cfg(QStringLiteral("trashrc"), KConfig::SimpleConfig);
     QVERIFY(cfg.hasGroup("Status"));
     QCOMPARE(cfg.group("Status").readEntry("Empty", true), false);
 
@@ -521,6 +548,45 @@ void FileUndoManagerTest::testTrashFiles()
     // We can't check that the trash is empty; other partitions might have their own trash
 }
 
+void FileUndoManagerTest::testRestoreTrashedFiles()
+{
+    if (!KProtocolInfo::isKnownProtocol(QStringLiteral("trash"))) {
+        QSKIP("kio_trash not installed");
+    }
+
+    // Trash it all at once: the file, the symlink, the subdir.
+    const QFile::Permissions origPerms = QFileInfo(srcFile()).permissions();
+    QList<QUrl> lst = sourceList();
+    lst.append(QUrl::fromLocalFile(srcSubDir()));
+    KIO::Job *job = KIO::trash(lst, KIO::HideProgressInfo);
+    job->setUiDelegate(0);
+    QVERIFY(job->exec());
+
+    const QMap<QString, QString> metaData = job->metaData();
+    QList<QUrl> trashUrls;
+    foreach(const QUrl &src, lst) {
+        QMap<QString, QString>::ConstIterator it = metaData.find("trashURL-" + src.path());
+        QVERIFY(it != metaData.constEnd());
+        trashUrls.append(QUrl(it.value()));
+    }
+
+    qDebug() << trashUrls;
+
+    // Restore from trash
+    KIO::RestoreJob *restoreJob = KIO::restoreFromTrash(trashUrls, KIO::HideProgressInfo);
+    restoreJob->setUiDelegate(0);
+    QVERIFY(restoreJob->exec());
+
+    QVERIFY(QFile::exists(srcFile()));
+    QCOMPARE(QFileInfo(srcFile()).permissions(), origPerms);
+#ifndef Q_OS_WIN
+    QVERIFY(QFileInfo(srcLink()).isSymLink());
+#endif
+    QVERIFY(QFile::exists(srcSubDir()));
+
+    // TODO support for RestoreJob in FileUndoManager !!!
+}
+
 static void setTimeStamp(const QString &path)
 {
 #ifdef Q_OS_UNIX
@@ -568,15 +634,14 @@ void FileUndoManagerTest::testPasteClipboardUndo()
     const QList<QUrl> urls(sourceList());
     QMimeData *mimeData = new QMimeData();
     mimeData->setUrls(urls);
-    mimeData->setData(QLatin1String("application/x-kde-cutselection"), "1");
+    KIO::setClipboardDataCut(mimeData, true);
     QClipboard *clipboard = QApplication::clipboard();
     clipboard->setMimeData(mimeData);
 
     // Paste the contents of the clipboard and check its status
     QUrl destDirUrl = QUrl::fromLocalFile(destDir());
-    KIO::CopyJob *job = qobject_cast<KIO::CopyJob *>(KIO::pasteClipboard(destDirUrl, 0, true));
+    KIO::Job *job = KIO::paste(mimeData, destDirUrl);
     QVERIFY(job);
-    FileUndoManager::self()->recordCopyJob(job);
     QVERIFY(job->exec());
 
     // Check if the clipboard was updated after paste operation
