@@ -31,6 +31,7 @@
 #include <QtCore/QDir>
 #include <QtCore/QHash>
 #include <QtCore/QVariant>
+#include <QtCore/QBuffer>
 #include <QUrl>
 
 #include <kprotocolinfo.h>
@@ -40,6 +41,9 @@
 #include <kio/deletejob.h>
 #include <kio/chmodjob.h>
 #include "kiotesthelper.h" // createTestFile etc.
+#ifndef Q_OS_WIN
+#include <unistd.h> // for readlink
+#endif
 
 QTEST_MAIN(JobTest)
 
@@ -51,7 +55,7 @@ static QString otherTmpDir()
     return QDir::tempPath() + "/jobtest/";
 #else
     // This one needs to be on another partition
-    return "/tmp/jobtest/";
+    return QStringLiteral("/tmp/jobtest/");
 #endif
 }
 
@@ -220,6 +224,68 @@ void JobTest::storedPut()
     QVERIFY(!spyPercent.isEmpty());
 }
 
+void JobTest::storedPutIODevice()
+{
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    QBuffer putData;
+    putData.setData("This is the put data");
+    QVERIFY(putData.open(QIODevice::ReadOnly));
+    KIO::TransferJob *job = KIO::storedPut(&putData, QUrl::fromLocalFile(filePath), 0600, KIO::Overwrite | KIO::HideProgressInfo);
+    QSignalSpy spyPercent(job, SIGNAL(percent(KJob*,ulong)));
+    QVERIFY(spyPercent.isValid());
+    QDateTime mtime = QDateTime::currentDateTime().addSecs(-30);   // 30 seconds ago
+    mtime.setTime_t(mtime.toTime_t()); // hack for losing the milliseconds
+    job->setModificationTime(mtime);
+    job->setUiDelegate(0);
+    QVERIFY(job->exec());
+    QFileInfo fileInfo(filePath);
+    QVERIFY(fileInfo.exists());
+    QCOMPARE(fileInfo.size(), (long long)putData.size());
+    QCOMPARE((int)fileInfo.permissions(), (int)(QFile::ReadOwner | QFile::WriteOwner | QFile::ReadUser | QFile::WriteUser));
+    QCOMPARE(fileInfo.lastModified(), mtime);
+    QVERIFY(!spyPercent.isEmpty());
+}
+
+void JobTest::storedPutIODeviceSlowDevice()
+{
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    const QUrl u = QUrl::fromLocalFile(filePath);
+    const QByteArray putDataContents = "This is the put data";
+    QBuffer putDataBuffer;
+    QVERIFY(putDataBuffer.open(QIODevice::ReadWrite));
+
+    KIO::StoredTransferJob *job = KIO::storedPut(&putDataBuffer, u, 0600, KIO::Overwrite | KIO::HideProgressInfo);
+    QSignalSpy spyPercent(job, SIGNAL(percent(KJob*,ulong)));
+    QVERIFY(spyPercent.isValid());
+    QDateTime mtime = QDateTime::currentDateTime().addSecs(-30);   // 30 seconds ago
+    mtime.setTime_t(mtime.toTime_t()); // hack for losing the milliseconds
+    job->setModificationTime(mtime);
+    job->setTotalSize(putDataContents.size());
+    job->setUiDelegate(0);
+    job->setAsyncDataEnabled(true);
+
+    QTimer t;
+    t.setSingleShot(true);
+    int size = -1;
+    connect(&t, &QTimer::timeout, this, [&putDataBuffer, &size, putDataContents]() {
+        putDataBuffer.write(putDataContents);
+        putDataBuffer.seek(0);
+//         qDebug() << "written" << putDataBuffer.size();
+        size = putDataBuffer.bytesAvailable();
+    });
+    t.start(200);
+    QVERIFY(job->exec());
+    QCOMPARE(size, putDataContents.size());
+    QCOMPARE(putDataBuffer.bytesAvailable(), 0);
+
+    QFileInfo fileInfo(filePath);
+    QVERIFY(fileInfo.exists());
+    QCOMPARE(fileInfo.size(), (long long)putDataContents.size());
+    QCOMPARE((int)fileInfo.permissions(), (int)(QFile::ReadOwner | QFile::WriteOwner | QFile::ReadUser | QFile::WriteUser));
+    QCOMPARE(fileInfo.lastModified(), mtime);
+    QVERIFY(!spyPercent.isEmpty());
+}
+
 ////
 
 void JobTest::copyLocalFile(const QString &src, const QString &dest)
@@ -227,13 +293,15 @@ void JobTest::copyLocalFile(const QString &src, const QString &dest)
     const QUrl u = QUrl::fromLocalFile(src);
     const QUrl d = QUrl::fromLocalFile(dest);
 
+    const int perms = 0666;
     // copy the file with file_copy
-    KIO::Job *job = KIO::file_copy(u, d, -1, KIO::HideProgressInfo);
+    KIO::Job *job = KIO::file_copy(u, d, perms, KIO::HideProgressInfo);
     job->setUiDelegate(0);
     bool ok = job->exec();
     QVERIFY(ok);
     QVERIFY(QFile::exists(dest));
     QVERIFY(QFile::exists(src));     // still there
+    QCOMPARE(int(QFileInfo(dest).permissions()), int(0x6666));
 
     {
         // check that the timestamp is the same (#24443)
@@ -329,6 +397,38 @@ void JobTest::copyLocalDirectory(const QString &src, const QString &_dest, int f
     QVERIFY(!ok);
 }
 
+#ifndef Q_OS_WIN
+static QString linkTarget(const QString &path)
+{
+    // Use readlink on Unix because symLinkTarget turns relative targets into absolute (#352927)
+    char linkTargetBuffer[4096];
+    const int n = readlink(QFile::encodeName(path).constData(), linkTargetBuffer, sizeof(linkTargetBuffer)-1);
+    if (n != -1) {
+        linkTargetBuffer[n] = 0;
+    }
+    return QFile::decodeName(linkTargetBuffer);
+}
+
+static void copyLocalSymlink(const QString &src, const QString &dest, const QString &expectedLinkTarget)
+{
+    QT_STATBUF buf;
+    QVERIFY(QT_LSTAT(QFile::encodeName(src).constData(), &buf) == 0);
+    QUrl u = QUrl::fromLocalFile(src);
+    QUrl d = QUrl::fromLocalFile(dest);
+
+    // copy the symlink
+    KIO::Job *job = KIO::copy(u, d, KIO::HideProgressInfo);
+    job->setUiDelegate(0);
+    job->setUiDelegateExtension(0);
+    QVERIFY2(job->exec(), qPrintable(job->error()));
+    QVERIFY(QT_LSTAT(QFile::encodeName(dest).constData(), &buf) == 0); // dest exists
+    QCOMPARE(linkTarget(dest), expectedLinkTarget);
+
+    // cleanup
+    QFile::remove(dest);
+}
+#endif
+
 void JobTest::copyFileToSamePartition()
 {
     const QString filePath = homeTmpDir() + "fileFromHome";
@@ -376,6 +476,68 @@ void JobTest::copyDirectoryToOtherPartition()
     copyLocalDirectory(src, dest);
 }
 
+void JobTest::copyRelativeSymlinkToSamePartition() // #352927
+{
+#ifdef Q_OS_WIN
+    QSKIP("Skipping symlink test on Windows");
+#else
+    const QString filePath = homeTmpDir() + "testlink";
+    const QString dest = homeTmpDir() + "testlink_copied";
+    createTestSymlink(filePath, "relative");
+    copyLocalSymlink(filePath, dest, QStringLiteral("relative"));
+    QFile::remove(filePath);
+#endif
+}
+
+void JobTest::copyAbsoluteSymlinkToOtherPartition()
+{
+#ifdef Q_OS_WIN
+    QSKIP("Skipping symlink test on Windows");
+#else
+    const QString filePath = homeTmpDir() + "testlink";
+    const QString dest = otherTmpDir() + "testlink_copied";
+    createTestSymlink(filePath, QFile::encodeName(homeTmpDir()));
+    copyLocalSymlink(filePath, dest, homeTmpDir());
+    QFile::remove(filePath);
+#endif
+}
+
+void JobTest::copyFolderWithUnaccessibleSubfolder()
+{
+    const QString src_dir = homeTmpDir() + "srcHome";
+    const QString dst_dir = homeTmpDir() + "dstHome";
+
+    QDir().remove(src_dir);
+    QDir().remove(dst_dir);
+
+    createTestDirectory(src_dir);
+    createTestDirectory(src_dir + "/folder1");
+    QString inaccessible = src_dir + "/folder1/inaccessible";
+    createTestDirectory(inaccessible);
+
+    QFile(inaccessible).setPermissions(QFile::Permissions()); // Make it inaccessible
+    //Copying should throw some warnings, as it cannot access some folders
+
+    KIO::CopyJob* job = KIO::copy(QUrl::fromLocalFile(src_dir), QUrl::fromLocalFile(dst_dir), KIO::HideProgressInfo);
+
+    QSignalSpy spy(job, SIGNAL(warning(KJob*,QString,QString)));
+    job->setUiDelegate( 0 ); // no skip dialog, thanks
+    QVERIFY(job->exec());
+
+    QFile(inaccessible).setPermissions(QFile::Permissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner));
+
+    KIO::DeleteJob* deljob1 = KIO::del(QUrl::fromLocalFile(src_dir), KIO::HideProgressInfo);
+    deljob1->setUiDelegate(0); // no skip dialog, thanks
+    QVERIFY(deljob1->exec());
+
+    KIO::DeleteJob* deljob2 = KIO::del(QUrl::fromLocalFile(dst_dir), KIO::HideProgressInfo);
+    deljob2->setUiDelegate(0); // no skip dialog, thanks
+    QVERIFY(deljob2->exec());
+
+    QCOMPARE(spy.count(), 1); // one warning should be emitted by the copy job
+
+}
+
 void JobTest::moveLocalFile(const QString &src, const QString &dest)
 {
     QVERIFY(QFile::exists(src));
@@ -383,12 +545,13 @@ void JobTest::moveLocalFile(const QString &src, const QString &dest)
     QUrl d = QUrl::fromLocalFile(dest);
 
     // move the file with file_move
-    KIO::Job *job = KIO::file_move(u, d, -1, KIO::HideProgressInfo);
+    KIO::Job *job = KIO::file_move(u, d, 0666, KIO::HideProgressInfo);
     job->setUiDelegate(0);
     bool ok = job->exec();
     QVERIFY(ok);
     QVERIFY(QFile::exists(dest));
     QVERIFY(!QFile::exists(src));     // not there anymore
+    QCOMPARE(int(QFileInfo(dest).permissions()), int(0x6666));
 
     // move it back with KIO::move()
     job = KIO::move(d, u, KIO::HideProgressInfo);
@@ -445,7 +608,7 @@ void JobTest::moveLocalDirectory(const QString &src, const QString &dest)
     job->setUiDelegate(0);
     job->setUiDelegateExtension(0);
     bool ok = job->exec();
-    QVERIFY(ok);
+    QVERIFY2(ok, qPrintable(job->errorString()));
     QVERIFY(QFile::exists(dest));
     QVERIFY(QFileInfo(dest).isDir());
     QVERIFY(QFileInfo(dest + "/testfile").isFile());
@@ -473,6 +636,22 @@ void JobTest::moveDirectoryToSamePartition()
     moveLocalDirectory(src, dest);
 }
 
+void JobTest::moveDirectoryIntoItself()
+{
+    qDebug();
+    const QString src = homeTmpDir() + "dirFromHome";
+    const QString dest = src + "/foo";
+    createTestDirectory(src);
+    QVERIFY(QFile::exists(src));
+    QUrl u = QUrl::fromLocalFile(src);
+    QUrl d = QUrl::fromLocalFile(dest);
+    KIO::CopyJob *job = KIO::move(u, d);
+    QVERIFY(!job->exec());
+    QCOMPARE(job->error(), (int)KIO::ERR_CANNOT_MOVE_INTO_ITSELF);
+    QCOMPARE(job->errorString(), i18n("A folder cannot be moved into itself"));
+    QDir(dest).removeRecursively();
+}
+
 void JobTest::moveFileToOtherPartition()
 {
     qDebug();
@@ -510,7 +689,7 @@ void JobTest::moveFileNoPermissions()
 #ifdef Q_OS_WIN
     qDebug() << "port to win32";
 #else
-    const QString src = "/etc/passwd";
+    const QString src = QStringLiteral("/etc/passwd");
     const QString dest = homeTmpDir() + "passwd";
     QVERIFY(QFile::exists(src));
     QVERIFY(QFileInfo(src).isFile());
@@ -540,9 +719,9 @@ void JobTest::moveDirectoryNoPermissions()
 #else
 
     // All of /etc is a bit much, so try to find something smaller:
-    QString src = "/etc/fonts";
+    QString src = QStringLiteral("/etc/fonts");
     if (!QFile::exists(src)) {
-        src = "/etc";
+        src = QStringLiteral("/etc");
     }
 
     const QString dest = homeTmpDir() + "mdnp";
@@ -595,7 +774,7 @@ void JobTest::listRecursive()
 #endif
                                       "fileFromHome,fileFromHome_copied");
 
-    const QString joinedNames = m_names.join(",");
+    const QString joinedNames = m_names.join(QStringLiteral(","));
     if (joinedNames.toLatin1() != ref_names) {
         qDebug("%s", qPrintable(joinedNames));
         qDebug("%s", ref_names.data());
@@ -684,7 +863,7 @@ void JobTest::directorySize()
 
 void JobTest::directorySizeError()
 {
-    KIO::DirectorySizeJob *job = KIO::directorySize(QUrl::fromLocalFile("/I/Dont/Exist"));
+    KIO::DirectorySizeJob *job = KIO::directorySize(QUrl::fromLocalFile(QStringLiteral("/I/Dont/Exist")));
     job->setUiDelegate(0);
     bool ok = job->exec();
     QVERIFY(!ok);
@@ -791,7 +970,7 @@ void JobTest::copyFileToSystem(bool resolve_local_urls)
 
 void JobTest::getInvalidUrl()
 {
-    QUrl url("http://strange<hostname>/");
+    QUrl url(QStringLiteral("http://strange<hostname>/"));
     QVERIFY(!url.isValid());
 
     KIO::SimpleJob *job = KIO::get(url, KIO::NoReload, KIO::HideProgressInfo);
@@ -915,13 +1094,21 @@ void JobTest::deleteManyDirs()
     deleteManyDirs(false);
 }
 
-static void createManyFiles(const QString &baseDir, int numFiles)
+static QList<QUrl> createManyFiles(const QString &baseDir, int numFiles)
 {
+    QList<QUrl> ret;
+    ret.reserve(numFiles);
     for (int i = 0; i < numFiles; ++i) {
         // create empty file
-        QFile f(baseDir + QString::number(i));
-        QVERIFY(f.open(QIODevice::WriteOnly));
+        const QString file = baseDir + QString::number(i);
+        QFile f(file);
+        bool ok = f.open(QIODevice::WriteOnly);
+        if (ok) {
+            f.write("Hello");
+            ret.append(QUrl::fromLocalFile(file));
+        }
     }
+    return ret;
 }
 
 void JobTest::deleteManyFilesIndependently()
@@ -930,13 +1117,15 @@ void JobTest::deleteManyFilesIndependently()
     dt.start();
     const int numFiles = 100; // Use 1000 for performance testing
     const QString baseDir = homeTmpDir();
-    createManyFiles(baseDir, numFiles);
+    const QList<QUrl> urls = createManyFiles(baseDir, numFiles);
+    QCOMPARE(urls.count(), numFiles);
     for (int i = 0; i < numFiles; ++i) {
         // delete each file independently. lots of jobs. this stress-tests kio scheduling.
-        const QString file = baseDir + QString::number(i);
+        const QUrl url = urls.at(i);
+        const QString file = url.toLocalFile();
         QVERIFY(QFile::exists(file));
         //qDebug() << file;
-        KIO::Job *job = KIO::del(QUrl::fromLocalFile(file), KIO::HideProgressInfo);
+        KIO::Job *job = KIO::del(url, KIO::HideProgressInfo);
         job->setUiDelegate(0);
         bool ok = job->exec();
         QVERIFY(ok);
@@ -954,13 +1143,8 @@ void JobTest::deleteManyFilesTogether(bool using_fast_path)
     dt.start();
     const int numFiles = 100; // Use 1000 for performance testing
     const QString baseDir = homeTmpDir();
-    createManyFiles(baseDir, numFiles);
-    QList<QUrl> urls;
-    for (int i = 0; i < numFiles; ++i) {
-        const QString file = baseDir + QString::number(i);
-        QVERIFY(QFile::exists(file));
-        urls.append(QUrl::fromLocalFile(file));
-    }
+    const QList<QUrl> urls = createManyFiles(baseDir, numFiles);
+    QCOMPARE(urls.count(), numFiles);
 
     //qDebug() << file;
     KIO::Job *job = KIO::del(urls, KIO::HideProgressInfo);
@@ -1065,7 +1249,7 @@ void JobTest::chmodFileError()
     mode_t newPerm = origPerm ^ S_IWGRP;
     QVERIFY(newPerm != origPerm);
     KFileItemList items; items << item;
-    KIO::Job *job = KIO::chmod(items, newPerm, S_IWGRP /*TODO: QFile::WriteGroup*/, QString("root"), QString(), false, KIO::HideProgressInfo);
+    KIO::Job *job = KIO::chmod(items, newPerm, S_IWGRP /*TODO: QFile::WriteGroup*/, QStringLiteral("root"), QString(), false, KIO::HideProgressInfo);
     // Simulate the user pressing "Skip" in the dialog.
     PredefinedAnswerJobUiDelegate extension;
     extension.m_skipResult = KIO::S_SKIP;
@@ -1177,10 +1361,10 @@ void JobTest::moveDestAlreadyExistsAutoRename(const QString &destDir, bool moveD
 {
     const QString prefix = moveDirs ? "dir " : "file ";
     QStringList sources;
-    const QString file1 = homeTmpDir() + prefix + "1";
-    const QString file2 = homeTmpDir() + prefix + "2";
-    const QString existingDest1 = destDir + prefix + "1";
-    const QString existingDest2 = destDir + prefix + "2";
+    const QString file1 = homeTmpDir() + prefix + "(1)";
+    const QString file2 = homeTmpDir() + prefix + "(2)";
+    const QString existingDest1 = destDir + prefix + "(1)";
+    const QString existingDest2 = destDir + prefix + "(2)";
     sources << file1 << file2 << existingDest1 << existingDest2;
     Q_FOREACH (const QString &source, sources) {
         if (moveDirs) {
@@ -1206,8 +1390,8 @@ void JobTest::moveDestAlreadyExistsAutoRename(const QString &destDir, bool moveD
     QVERIFY(!QFile::exists(file2)); // it was moved
     QVERIFY(QFile::exists(existingDest1));
     QVERIFY(QFile::exists(existingDest2));
-    const QString file3 = destDir + prefix + "3";
-    const QString file4 = destDir + prefix + "4";
+    const QString file3 = destDir + prefix + "(3)";
+    const QString file4 = destDir + prefix + "(4)";
     QVERIFY(QFile::exists(file3));
     QVERIFY(QFile::exists(file4));
     if (moveDirs) {
@@ -1289,3 +1473,43 @@ void JobTest::moveOverSymlinkToSelf() // #169547
 #endif
 }
 
+void JobTest::multiGet()
+{
+    const int numFiles = 10;
+    const QString baseDir = homeTmpDir();
+    const QList<QUrl> urls = createManyFiles(baseDir, numFiles);
+    QCOMPARE(urls.count(), numFiles);
+
+    //qDebug() << file;
+    KIO::MultiGetJob *job = KIO::multi_get(0, urls.at(0), KIO::MetaData()); // TODO: missing KIO::HideProgressInfo
+    QSignalSpy spyData(job, SIGNAL(data(long, QByteArray)));
+    QSignalSpy spyMimeType(job, SIGNAL(mimetype(long, QString)));
+    QSignalSpy spyResultId(job, SIGNAL(result(long)));
+    QSignalSpy spyResult(job, SIGNAL(result(KJob*)));
+    job->setUiDelegate(0);
+
+    for (int i = 1; i < numFiles; ++i) {
+        const QUrl url = urls.at(i);
+        job->get(i, url, KIO::MetaData());
+    }
+    //connect(job, &KIO::MultiGetJob::result, [=] (long id) { qDebug() << "ID I got" << id;});
+    //connect(job, &KJob::result, [this](KJob* ) {qDebug() << "END";});
+
+    bool ok = job->exec();
+    QVERIFY(ok);
+
+    QCOMPARE(spyResult.count(), 1);
+    QCOMPARE(spyResultId.count(), numFiles);
+    QCOMPARE(spyMimeType.count(), numFiles);
+    QCOMPARE(spyData.count(), numFiles * 2);
+    for (int i = 0; i < numFiles; ++i) {
+        QCOMPARE(spyResultId.at(i).at(0).toInt(), i);
+        QCOMPARE(spyMimeType.at(i).at(0).toInt(), i);
+        QCOMPARE(spyMimeType.at(i).at(1).toString(), QString("text/plain"));
+        QCOMPARE(spyData.at(i * 2).at(0).toInt(), i);
+        QCOMPARE(QString(spyData.at(i * 2).at(1).toByteArray()), QString("Hello"));
+        QCOMPARE(spyData.at(i * 2 + 1).at(0).toInt(), i);
+        QCOMPARE(QString(spyData.at(i * 2 + 1).at(1).toByteArray()), QLatin1String(""));
+    }
+}
+
