@@ -39,11 +39,16 @@ private Q_SLOTS:
     void storedGet();
     void put();
     void storedPut();
+    void storedPutIODevice();
+    void storedPutIODeviceSlowDevice();
     void copyFileToSamePartition();
     void copyDirectoryToSamePartition();
     void copyDirectoryToExistingDirectory();
     void copyFileToOtherPartition();
     void copyDirectoryToOtherPartition();
+    void copyRelativeSymlinkToSamePartition();
+    void copyAbsoluteSymlinkToOtherPartition();
+    void copyFolderWithUnaccessibleSubfolder();
     void listRecursive();
     void listFile();
     void killJob();
@@ -53,6 +58,7 @@ private Q_SLOTS:
     void directorySizeError();
     void moveFileToSamePartition();
     void moveDirectoryToSamePartition();
+    void moveDirectoryIntoItself();
     void moveFileToOtherPartition();
     void moveSymlinkToOtherPartition();
     void moveDirectoryToOtherPartition();
