@@ -23,7 +23,6 @@
 
 #include "kiocore_export.h"
 #include <QObject>
-#include <QProcess> // Q_PID
 #include <QDateTime>
 #include <QUrl>
 #include <QScopedPointer>
@@ -50,7 +49,7 @@ public:
 
     bool match(const QString &protocol, const QString &host, bool connected) const;
     void connect(const QString &app_socket);
-    Q_PID pid() const;
+    qint64 pid() const;
     int age(const QDateTime &now) const;
     void reparseConfiguration();
     bool onHold(const QUrl &url) const;
