@@ -22,6 +22,7 @@
 #include <klocalizedstring.h>
 #include <QUrl>
 #include <QDateTime>
+#include <QDataStream>
 #include <sys/stat.h> // S_IRUSR etc
 
 QString KIO::Job::errorString() const
@@ -232,6 +233,12 @@ KIOCORE_EXPORT QString KIO::buildErrorString(int errorCode, const QString &error
     case KIO::ERR_POST_NO_SIZE:
         result = i18n("The required content size information was not provided for a POST operation.");
         break;
+    case KIO::ERR_DROP_ON_ITSELF:
+        result = i18n("A file or folder cannot be dropped onto itself");
+        break;
+    case KIO::ERR_CANNOT_MOVE_INTO_ITSELF:
+        result = i18n("A folder cannot be moved into itself");
+        break;
     default:
         result = i18n("Unknown error code %1\n%2\nPlease send a full bug report at http://bugs.kde.org.",  errorCode,  errorText);
         break;
@@ -267,7 +274,7 @@ QStringList KIO::Job::detailedErrorStrings(const QUrl *reqUrl /*= 0*/,
     ret << i18nc("@info %1 error name, %2 description",
                  "<qt><p><b>%1</b></p><p>%2</p></qt>", errorName, description);
 
-    ret2 = QLatin1String("<qt>");
+    ret2 = QStringLiteral("<qt>");
     if (!techName.isEmpty())
         ret2 += QLatin1String("<p>") + i18n("<b>Technical reason</b>: ") +
                 techName + QLatin1String("</p>");
@@ -281,12 +288,12 @@ QStringList KIO::Job::detailedErrorStrings(const QUrl *reqUrl /*= 0*/,
             QLatin1String("</ul>");
     if (!causes.isEmpty()) {
         ret2 += QLatin1String("<p>") + i18n("<b>Possible causes</b>:") +
-                QLatin1String("</p><ul><li>") + causes.join("</li><li>") +
+                QLatin1String("</p><ul><li>") + causes.join(QStringLiteral("</li><li>")) +
                 QLatin1String("</li></ul>");
     }
     if (!solutions.isEmpty()) {
         ret2 += QLatin1String("<p>") + i18n("<b>Possible solutions</b>:") +
-                QLatin1String("</p><ul><li>") + solutions.join("</li><li>") +
+                QLatin1String("</p><ul><li>") + solutions.join(QStringLiteral("</li><li>")) +
                 QLatin1String("</li></ul>");
     }
     ret2 += QLatin1String("</qt>");
@@ -315,7 +322,7 @@ KIOCORE_EXPORT QByteArray KIO::rawErrorDetail(int errorCode, const QString &erro
         path = reqUrl->path();
 
         // detect if protocol is a network protocol...
-        isSlaveNetwork = KProtocolInfo::protocolClass(protocol) == ":internet";
+        isSlaveNetwork = KProtocolInfo::protocolClass(protocol) == QLatin1String(":internet");
     } else {
         // assume that the errorText has the location we are interested in
         url = host = domain = path = filename = errorText;
@@ -1018,12 +1025,26 @@ KIOCORE_EXPORT QByteArray KIO::rawErrorDetail(int errorCode, const QString &erro
         solutions << i18n("Choose a different filename for the destination file.");
         break;
 
+    case KIO::ERR_DROP_ON_ITSELF:
+        errorName = i18n("File or Folder dropped onto itself");
+        description = i18n("The operation could not be completed because the "
+                           "source and destination file or folder are the same.");
+        solutions << i18n("Drop the item into a different file or folder.");
+        break;
+
     // We assume that the slave has all the details
     case KIO::ERR_SLAVE_DEFINED:
         errorName.clear();
         description = errorText;
         break;
 
+    case KIO::ERR_CANNOT_MOVE_INTO_ITSELF:
+        errorName = i18n("Folder moved into itself");
+        description = i18n("The operation could not be completed because the "
+                           "source can not be moved into itself.");
+        solutions << i18n("Move the item into a different folder.");
+        break;
+
     default:
         // fall back to the plain error...
         errorName = i18n("Undocumented Error");
