@@ -27,6 +27,7 @@
 #include <QtCore/QDirIterator>
 #include <QtCore/QFile>
 #include <QtCore/QMap>
+#include <QtCore/QDataStream>
 #include <QMimeDatabase>
 #include <QDebug>
 #include <qplatformdefs.h>
@@ -228,8 +229,8 @@ void KFileItemPrivate::readUDSEntry(bool _urlIsDirectory)
 {
     // extract fields from the KIO::UDS Entry
 
-    m_fileMode = m_entry.numberValue(KIO::UDSEntry::UDS_FILE_TYPE);
-    m_permissions = m_entry.numberValue(KIO::UDSEntry::UDS_ACCESS);
+    m_fileMode = m_entry.numberValue(KIO::UDSEntry::UDS_FILE_TYPE, KFileItem::Unknown);
+    m_permissions = m_entry.numberValue(KIO::UDSEntry::UDS_ACCESS, KFileItem::Unknown);
     m_strName = m_entry.stringValue(KIO::UDSEntry::UDS_NAME);
 
     const QString displayName = m_entry.stringValue(KIO::UDSEntry::UDS_DISPLAY_NAME);
@@ -730,13 +731,17 @@ QMimeType KFileItem::determineMimeType() const
 
     if (!d->m_mimeType.isValid() || !d->m_bMimeTypeKnown) {
         QMimeDatabase db;
-        bool isLocalUrl;
-        const QUrl url = mostLocalUrl(&isLocalUrl);
-        d->m_mimeType = db.mimeTypeForUrl(url);
-        // was:  d->m_mimeType = KMimeType::findByUrl( url, d->m_fileMode, isLocalUrl );
-        // => we are no longer using d->m_fileMode for remote URLs.
-        Q_ASSERT(d->m_mimeType.isValid());
-        //qDebug() << d << "finding final mimetype for" << url << ":" << d->m_mimeType.name();
+        if (isDir()) {
+            d->m_mimeType = db.mimeTypeForName(QStringLiteral("inode/directory"));
+        } else {
+            bool isLocalUrl;
+            const QUrl url = mostLocalUrl(&isLocalUrl);
+            d->m_mimeType = db.mimeTypeForUrl(url);
+            // was:  d->m_mimeType = KMimeType::findByUrl( url, d->m_fileMode, isLocalUrl );
+            // => we are no longer using d->m_fileMode for remote URLs.
+            Q_ASSERT(d->m_mimeType.isValid());
+            //qDebug() << d << "finding final mimetype for" << url << ":" << d->m_mimeType.name();
+        }
         d->m_bMimeTypeKnown = true;
     }
 
@@ -805,7 +810,7 @@ QString KFileItem::mimeComment() const
     QMimeType mime = currentMimeType();
     // This cannot move to kio_file (with UDS_DISPLAY_TYPE) because it needs
     // the mimetype to be determined, which is done here, and possibly delayed...
-    if (isLocalUrl && !d->isSlow() && mime.inherits("application/x-desktop")) {
+    if (isLocalUrl && !d->isSlow() && mime.inherits(QStringLiteral("application/x-desktop"))) {
         KDesktopFile cfg(url.toLocalFile());
         QString comment = cfg.desktopGroup().readEntry("Comment");
         if (!comment.isEmpty()) {
@@ -816,7 +821,7 @@ QString KFileItem::mimeComment() const
     // Support for .directory file in directories
     if (isLocalUrl && isDir() && isDirectoryMounted(url)) {
         QUrl u(url);
-        u.setPath(u.path() + QString::fromLatin1("/.directory"));
+        u.setPath(u.path() + QLatin1String("/.directory"));
         const KDesktopFile cfg(u.toLocalFile());
         const QString comment = cfg.readComment();
         if (!comment.isEmpty()) {
@@ -835,7 +840,7 @@ QString KFileItem::mimeComment() const
 
 static QString iconFromDirectoryFile(const QString &path)
 {
-    const QString filePath = path + QString::fromLatin1("/.directory");
+    const QString filePath = path + QLatin1String("/.directory");
     if (!QFileInfo(filePath).isFile()) { // exists -and- is a file
         return QString();
     }
@@ -882,7 +887,7 @@ static QString iconFromDesktopFile(const QString &path)
             if (url.scheme() == QLatin1String("trash")) {
                 // We need to find if the trash is empty, preferably without using a KIO job.
                 // So instead kio_trash leaves an entry in its config file for us.
-                KConfig trashConfig("trashrc", KConfig::SimpleConfig);
+                KConfig trashConfig(QStringLiteral("trashrc"), KConfig::SimpleConfig);
                 if (trashConfig.group("Status").readEntry("Empty", true)) {
                     return emptyIcon;
                 }
@@ -892,6 +897,42 @@ static QString iconFromDesktopFile(const QString &path)
     return icon;
 }
 
+static QMap<QString, QString> standardLocationsMap()
+{
+    static const
+            struct { QStandardPaths::StandardLocation location;
+                     QString name; }
+                   mapping[] = {
+            { QStandardPaths::MusicLocation, QStringLiteral("folder-music") },
+            { QStandardPaths::MoviesLocation, QStringLiteral("folder-videos") },
+            { QStandardPaths::PicturesLocation, QStringLiteral("folder-pictures") },
+            { QStandardPaths::TempLocation, QStringLiteral("folder-temp") },
+            { QStandardPaths::DownloadLocation, QStringLiteral("folder-download") },
+            // Order matters here as paths can be reused for multiple purposes
+            // We essentially want more generic choices to trump more specific
+            // ones.
+            // home > desktop > documents > *.
+            { QStandardPaths::DocumentsLocation, QStringLiteral("folder-documents") },
+            { QStandardPaths::DesktopLocation, QStringLiteral("user-desktop") },
+            { QStandardPaths::HomeLocation, QStringLiteral("user-home") } };
+    static const int count = sizeof mapping / sizeof *mapping;
+
+    QMap<QString, QString> map;
+    for (int i = 0 ; i < count; ++i) {
+        auto locations = QStandardPaths::standardLocations(mapping[i].location);
+        Q_FOREACH(const QString &location, locations) {
+            map.insert(location, mapping[i].name);
+        }
+    }
+    return map;
+}
+
+static QString iconForStandardPath(const QString &localDirectory)
+{
+    static auto map = standardLocationsMap();
+    return map.value(localDirectory, QString());
+}
+
 QString KFileItem::iconName() const
 {
     if (!d) {
@@ -922,7 +963,7 @@ QString KFileItem::iconName() const
 
     const bool delaySlowOperations = d->m_delayedMimeTypes;
 
-    if (isLocalUrl && !delaySlowOperations && mime.inherits("application/x-desktop")) {
+    if (isLocalUrl && !delaySlowOperations && mime.inherits(QStringLiteral("application/x-desktop"))) {
         d->m_iconName = iconFromDesktopFile(url.toLocalFile());
         if (!d->m_iconName.isEmpty()) {
             d->m_useIconNameCache = d->m_bMimeTypeKnown;
@@ -930,8 +971,16 @@ QString KFileItem::iconName() const
         }
     }
 
-    if (isLocalUrl && !delaySlowOperations && isDir() && isDirectoryMounted(url)) {
-        d->m_iconName = iconFromDirectoryFile(url.toLocalFile());
+    if (isLocalUrl && !delaySlowOperations && isDir()) {
+        if (isDirectoryMounted(url)) {
+            d->m_iconName = iconFromDirectoryFile(url.toLocalFile());
+            if (!d->m_iconName.isEmpty()) {
+                d->m_useIconNameCache = d->m_bMimeTypeKnown;
+                return d->m_iconName;
+            }
+        }
+
+        d->m_iconName = iconForStandardPath(url.toLocalFile());
         if (!d->m_iconName.isEmpty()) {
             d->m_useIconNameCache = d->m_bMimeTypeKnown;
             return d->m_iconName;
@@ -968,7 +1017,7 @@ static bool checkDesktopFile(const KFileItem &item, bool _determineMimeType)
 
     // return true if desktop file
     QMimeType mime = _determineMimeType ? item.determineMimeType() : item.currentMimeType();
-    return mime.inherits("application/x-desktop");
+    return mime.inherits(QStringLiteral("application/x-desktop"));
 }
 
 QStringList KFileItem::overlays() const
@@ -979,12 +1028,11 @@ QStringList KFileItem::overlays() const
 
     QStringList names = d->m_entry.stringValue(KIO::UDSEntry::UDS_ICON_OVERLAY_NAMES).split(',');
     if (d->m_bLink) {
-        names.append("emblem-symbolic-link");
+        names.append(QStringLiteral("emblem-symbolic-link"));
     }
 
-    if (((d->m_fileMode & QT_STAT_MASK) != QT_STAT_DIR)  // Locked dirs have a special icon, use the overlay for files only
-            && !isReadable()) {
-        names.append("object-locked");
+    if (!isReadable()) {
+        names.append(QStringLiteral("object-locked"));
     }
 
     if (checkDesktopFile(*this, false)) {
@@ -994,7 +1042,7 @@ QStringList KFileItem::overlays() const
         // Add a warning emblem if this is an executable desktop file
         // which is untrusted.
         if (group.hasKey("Exec") && !KDesktopFile::isAuthorizedDesktopFile(localPath())) {
-            names.append("emblem-important");
+            names.append(QStringLiteral("emblem-important"));
         }
 
         if (cfg.hasDeviceType()) {
@@ -1002,14 +1050,14 @@ QStringList KFileItem::overlays() const
             if (!dev.isEmpty()) {
                 KMountPoint::Ptr mountPoint = KMountPoint::currentMountPoints().findByDevice(dev);
                 if (mountPoint) { // mounted?
-                    names.append("emblem-mounted");
+                    names.append(QStringLiteral("emblem-mounted"));
                 }
             }
         }
     }
 
     if (isHidden()) {
-        names.append("hidden");
+        names.append(QStringLiteral("hidden"));
     }
 
 #ifndef Q_OS_WIN
@@ -1017,16 +1065,11 @@ QStringList KFileItem::overlays() const
         if (KSambaShare::instance()->isDirectoryShared(d->m_url.toLocalFile()) ||
                 KNFSShare::instance()->isDirectoryShared(d->m_url.toLocalFile())) {
             //qDebug() << d->m_url.path();
-            names.append("network-workgroup");
+            names.append(QStringLiteral("network-workgroup"));
         }
     }
 #endif  // Q_OS_WIN
 
-    if (d->m_url.path().endsWith(QLatin1String(".gz")) &&
-            d->m_mimeType.inherits("application/x-gzip")) {
-        names.append("application-zip");
-    }
-
     return names;
 }
 
@@ -1122,6 +1165,13 @@ bool KFileItem::isHidden() const
     return fileName.length() > 1 && fileName[0] == '.';  // Just "." is current directory, not hidden.
 }
 
+void KFileItem::setHidden()
+{
+    if (d) {
+        d->m_hidden = KFileItemPrivate::Hidden;
+    }
+}
+
 bool KFileItem::isDir() const
 {
     if (!d) {
@@ -1145,7 +1195,7 @@ bool KFileItem::isFile() const
     return !isDir();
 }
 
-#ifndef KDE_NO_DEPRECATED
+#ifndef KIOCORE_NO_DEPRECATED
 bool KFileItem::acceptsDrops() const
 {
     // A directory ?
@@ -1158,7 +1208,7 @@ bool KFileItem::acceptsDrops() const
         return false;
     }
 
-    if (mimetype() == "application/x-desktop") {
+    if (mimetype() == QLatin1String("application/x-desktop")) {
         return true;
     }
 
@@ -1190,9 +1240,9 @@ QString KFileItem::getStatusBarInfo() const
     } else if (targetUrl() != url()) {
         text += i18n(" (Points to %1)", targetUrl().toDisplayString());
     } else if ((d->m_fileMode & QT_STAT_MASK) == QT_STAT_REG) {
-        text += QString(" (%1, %2)").arg(comment, KIO::convertSize(size()));
+        text += QStringLiteral(" (%1, %2)").arg(comment, KIO::convertSize(size()));
     } else {
-        text += QString(" (%1)").arg(comment);
+        text += QStringLiteral(" (%1)").arg(comment);
     }
     return text;
 }
@@ -1256,7 +1306,7 @@ QString KFileItem::timeString(FileTimes which) const
     return d->time(which).toString();
 }
 
-#ifndef KDE_NO_DEPRECATED
+#ifndef KIOCORE_NO_DEPRECATED
 QString KFileItem::timeString(unsigned int which) const
 {
     if (!d) {
@@ -1275,7 +1325,7 @@ QString KFileItem::timeString(unsigned int which) const
 }
 #endif
 
-#ifndef KDE_NO_DEPRECATED
+#ifndef KIOCORE_NO_DEPRECATED
 void KFileItem::assign(const KFileItem &item)
 {
     *this = item;
@@ -1452,11 +1502,15 @@ QMimeType KFileItem::currentMimeType() const
         // On-demand fast (but not always accurate) mimetype determination
         Q_ASSERT(!d->m_url.isEmpty());
         QMimeDatabase db;
+        if (isDir()) {
+            d->m_mimeType = db.mimeTypeForName(QStringLiteral("inode/directory"));
+            return d->m_mimeType;
+        }
         const QUrl url = mostLocalUrl();
         if (d->m_delayedMimeTypes) {
             const QList<QMimeType> mimeTypes = db.mimeTypesForFileName(url.path());
             if (mimeTypes.isEmpty()) {
-                d->m_mimeType = db.mimeTypeForName("application/octet-stream");
+                d->m_mimeType = db.mimeTypeForName(QStringLiteral("application/octet-stream"));
                 d->m_bMimeTypeKnown = false;
             } else {
                 d->m_mimeType = mimeTypes.first();
