@@ -19,7 +19,6 @@
 */
 #include "kfileplacesmodel.h"
 #include "kfileplacesitem_p.h"
-#include "kfileplacessharedbookmarks_p.h"
 
 #ifdef _WIN32_WCE
 #include "Windows.h"
@@ -60,22 +59,20 @@
 class KFilePlacesModel::Private
 {
 public:
-    Private(KFilePlacesModel *self) : q(self), bookmarkManager(0), sharedBookmarks(0) {}
+    Private(KFilePlacesModel *self) : q(self), bookmarkManager(0) {}
     ~Private()
     {
-        delete sharedBookmarks;
         qDeleteAll(items);
     }
 
     KFilePlacesModel *q;
 
     QList<KFilePlacesItem *> items;
-    QSet<QString> availableDevices;
+    QVector<QString> availableDevices;
     QMap<QObject *, QPersistentModelIndex> setupInProgress;
 
     Solid::Predicate predicate;
     KBookmarkManager *bookmarkManager;
-    KFilePlacesSharedBookmarks *sharedBookmarks;
 
     void reloadAndSignal();
     QList<KFilePlacesItem *> loadBookmarkList();
@@ -92,17 +89,10 @@ public:
 KFilePlacesModel::KFilePlacesModel(QObject *parent)
     : QAbstractItemModel(parent), d(new Private(this))
 {
-    const QString file = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1Char('/') + "kfileplaces/bookmarks.xml";
-    d->bookmarkManager = KBookmarkManager::managerForFile(file, "kfilePlaces");
+    const QString file = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + "/user-places.xbel";
+    d->bookmarkManager = KBookmarkManager::managerForExternalFile(file);
 
-    // Let's put some places in there if it's empty. We have a corner case here:
-    // Given you have bookmarked some folders (which have been saved on
-    // ~/.local/share/user-places.xbel (according to freedesktop bookmarks spec), and
-    // deleted the home directory ~/.kde, the call managerForFile() will return the
-    // bookmark manager for the fallback "kfilePlaces", making root.first().isNull() being
-    // false (you have your own items bookmarked), resulting on only being added your own
-    // bookmarks, and not the default ones too. So, we also check if kfileplaces/bookmarks.xml
-    // file exists, and if it doesn't, we also add the default places. (ereslibre)
+    // Let's put some places in there if it's empty.
     KBookmarkGroup root = d->bookmarkManager->root();
     if (root.first().isNull() || !QFile::exists(file)) {
 
@@ -115,11 +105,11 @@ KFilePlacesModel::KFilePlacesModel(QObject *parent)
         // (coles, 13th May 2009)
 
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              "Home", I18N_NOOP2("KFile System Bookmarks", "Home"),
-                                              QUrl::fromLocalFile(QDir::homePath()), "user-home");
+                                              QStringLiteral("Home"), I18N_NOOP2("KFile System Bookmarks", "Home"),
+                                              QUrl::fromLocalFile(QDir::homePath()), QStringLiteral("user-home"));
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              "Network", I18N_NOOP2("KFile System Bookmarks", "Network"),
-                                              QUrl("remote:/"), "network-workgroup");
+                                              QStringLiteral("Network"), I18N_NOOP2("KFile System Bookmarks", "Network"),
+                                              QUrl(QStringLiteral("remote:/")), QStringLiteral("network-workgroup"));
 #if defined(_WIN32_WCE)
         // adding drives
         foreach (const QFileInfo &info, QDir::drives()) {
@@ -130,33 +120,28 @@ KFilePlacesModel::KFilePlacesModel(QObject *parent)
         }
 #elif !defined(Q_OS_WIN)
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              "Root", I18N_NOOP2("KFile System Bookmarks", "Root"),
-                                              QUrl::fromLocalFile("/"), "folder-red");
+                                              QStringLiteral("Root"), I18N_NOOP2("KFile System Bookmarks", "Root"),
+                                              QUrl::fromLocalFile(QStringLiteral("/")), QStringLiteral("folder-red"));
 #endif
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              "Trash", I18N_NOOP2("KFile System Bookmarks", "Trash"),
-                                              QUrl("trash:/"), "user-trash");
+                                              QStringLiteral("Trash"), I18N_NOOP2("KFile System Bookmarks", "Trash"),
+                                              QUrl(QStringLiteral("trash:/")), QStringLiteral("user-trash"));
 
         // Force bookmarks to be saved. If on open/save dialog and the bookmarks are not saved, QFile::exists
         // will always return false, which opening/closing all the time the open/save dialog would case the
-        // bookmarks to be added once each time, having lots of times each bookmark. This forces the defaults
-        // to be saved on the bookmarks.xml file. Of course, the complete list of bookmarks (those that come from
-        // user-places.xbel will be filled later). (ereslibre)
+        // bookmarks to be added once each time, having lots of times each bookmark. (ereslibre)
         d->bookmarkManager->saveAs(file);
     }
 
-    // create after, so if we have own places, they are added afterwards, in case of equal priorities
-    d->sharedBookmarks = new KFilePlacesSharedBookmarks(d->bookmarkManager);
-
-    QString predicate("[[[[ StorageVolume.ignored == false AND [ StorageVolume.usage == 'FileSystem' OR StorageVolume.usage == 'Encrypted' ]]"
+    QString predicate(QString::fromLatin1("[[[[ StorageVolume.ignored == false AND [ StorageVolume.usage == 'FileSystem' OR StorageVolume.usage == 'Encrypted' ]]"
                       " OR "
                       "[ IS StorageAccess AND StorageDrive.driveType == 'Floppy' ]]"
                       " OR "
                       "OpticalDisc.availableContent & 'Audio' ]"
                       " OR "
-                      "StorageAccess.ignored == false ]");
+                      "StorageAccess.ignored == false ]"));
 
-    if (KProtocolInfo::isKnownProtocol("mtp")) {
+    if (KProtocolInfo::isKnownProtocol(QStringLiteral("mtp"))) {
         predicate.prepend("[");
         predicate.append(" OR PortableMediaPlayer.supportedProtocols == 'mtp']");
     }
@@ -181,7 +166,7 @@ KFilePlacesModel::~KFilePlacesModel()
 
 QUrl KFilePlacesModel::url(const QModelIndex &index) const
 {
-    return QUrl(data(index, UrlRole).toUrl());
+    return data(index, UrlRole).toUrl();
 }
 
 bool KFilePlacesModel::setupNeeded(const QModelIndex &index) const
@@ -348,8 +333,9 @@ void KFilePlacesModel::Private::_k_deviceAdded(const QString &udi)
 
 void KFilePlacesModel::Private::_k_deviceRemoved(const QString &udi)
 {
-    if (availableDevices.contains(udi)) {
-        availableDevices.remove(udi);
+    auto it = std::find(availableDevices.begin(), availableDevices.end(), udi);
+    if (it != availableDevices.end()) {
+        availableDevices.erase(it);
         _k_reloadBookmarks();
     }
 }
@@ -442,12 +428,16 @@ QList<KFilePlacesItem *> KFilePlacesModel::Private::loadBookmarkList()
 
     KBookmarkGroup root = bookmarkManager->root();
     KBookmark bookmark = root.first();
-    QSet<QString> devices = availableDevices;
+    QVector<QString> devices = availableDevices;
 
     while (!bookmark.isNull()) {
-        QString udi = bookmark.metaDataItem("UDI");
-        QString appName = bookmark.metaDataItem("OnlyInApp");
-        bool deviceAvailable = devices.remove(udi);
+        QString udi = bookmark.metaDataItem(QStringLiteral("UDI"));
+        QString appName = bookmark.metaDataItem(QStringLiteral("OnlyInApp"));
+        auto it = std::find(devices.begin(), devices.end(), udi);
+        bool deviceAvailable = (it != devices.end());
+        if (it != devices.end()) {
+            devices.erase(it);
+        }
 
         bool allowedHere = appName.isEmpty() || (appName == QCoreApplication::instance()->applicationName());
 
@@ -510,14 +500,14 @@ Qt::ItemFlags KFilePlacesModel::flags(const QModelIndex &index) const
 
 static QString _k_internalMimetype(const KFilePlacesModel *const self)
 {
-    return QString("application/x-kfileplacesmodel-") + QString::number((qptrdiff)self);
+    return QStringLiteral("application/x-kfileplacesmodel-") + QString::number(reinterpret_cast<qptrdiff>(self));
 }
 
 QStringList KFilePlacesModel::mimeTypes() const
 {
     QStringList types;
 
-    types << _k_internalMimetype(this) << "text/uri-list";
+    types << _k_internalMimetype(this) << QStringLiteral("text/uri-list");
 
     return types;
 }
@@ -612,7 +602,7 @@ bool KFilePlacesModel::dropMimeData(const QMimeData *data, Qt::DropAction action
         // adjust if necessary.
         d->items.move(itemRow, itemRow < destRow ? (destRow - 1) : destRow);
         endMoveRows();
-    } else if (data->hasFormat("text/uri-list")) {
+    } else if (data->hasFormat(QStringLiteral("text/uri-list"))) {
         // The operation is an add
         const QList<QUrl> urls = KUrlMimeData::urlsFromMimeData(data);
 
@@ -624,7 +614,7 @@ bool KFilePlacesModel::dropMimeData(const QMimeData *data, Qt::DropAction action
 
             QString mimeString;
             if (!job->exec()) {
-                mimeString = QLatin1String("unknown");
+                mimeString = QStringLiteral("unknown");
             } else {
                 mimeString = job->mimetype();
             }
@@ -636,7 +626,7 @@ bool KFilePlacesModel::dropMimeData(const QMimeData *data, Qt::DropAction action
                 continue;
             }
 
-            if (!mimetype.inherits("inode/directory")) {
+            if (!mimetype.inherits(QStringLiteral("inode/directory"))) {
                 // Only directories are allowed
                 continue;
             }
@@ -675,7 +665,7 @@ void KFilePlacesModel::addPlace(const QString &text, const QUrl &url,
                          text, url, iconName);
 
     if (!appName.isEmpty()) {
-        bookmark.setMetaDataItem("OnlyInApp", appName);
+        bookmark.setMetaDataItem(QStringLiteral("OnlyInApp"), appName);
     }
 
     if (after.isValid()) {
@@ -708,7 +698,7 @@ void KFilePlacesModel::editPlace(const QModelIndex &index, const QString &text,
     bookmark.setFullText(text);
     bookmark.setUrl(url);
     bookmark.setIcon(iconName);
-    bookmark.setMetaDataItem("OnlyInApp", appName);
+    bookmark.setMetaDataItem(QStringLiteral("OnlyInApp"), appName);
 
     d->reloadAndSignal();
     emit dataChanged(index, index);
@@ -750,7 +740,7 @@ void KFilePlacesModel::setPlaceHidden(const QModelIndex &index, bool hidden)
         return;
     }
 
-    bookmark.setMetaDataItem("IsHidden", (hidden ? "true" : "false"));
+    bookmark.setMetaDataItem(QStringLiteral("IsHidden"), (hidden ? "true" : "false"));
 
     d->reloadAndSignal();
     emit dataChanged(index, index);
@@ -792,16 +782,16 @@ QAction *KFilePlacesModel::teardownActionForIndex(const QModelIndex &index) cons
 
         QString iconName;
         QString text;
-        QString label = data(index, Qt::DisplayRole).toString().replace('&', "&&");
+        QString label = data(index, Qt::DisplayRole).toString().replace('&', QLatin1String("&&"));
 
         if (device.is<Solid::OpticalDisc>()) {
             text = i18n("&Release '%1'", label);
         } else if (removable || hotpluggable) {
             text = i18n("&Safely Remove '%1'", label);
-            iconName = "media-eject";
+            iconName = QStringLiteral("media-eject");
         } else {
             text = i18n("&Unmount '%1'", label);
-            iconName = "media-eject";
+            iconName = QStringLiteral("media-eject");
         }
 
         if (!iconName.isEmpty()) {
@@ -820,10 +810,10 @@ QAction *KFilePlacesModel::ejectActionForIndex(const QModelIndex &index) const
 
     if (device.is<Solid::OpticalDisc>()) {
 
-        QString label = data(index, Qt::DisplayRole).toString().replace('&', "&&");
+        QString label = data(index, Qt::DisplayRole).toString().replace('&', QLatin1String("&&"));
         QString text = i18n("&Eject '%1'", label);
 
-        return new QAction(QIcon::fromTheme("media-eject"), text, 0);
+        return new QAction(QIcon::fromTheme(QStringLiteral("media-eject")), text, 0);
     }
 
     return 0;
@@ -854,7 +844,7 @@ void KFilePlacesModel::requestEject(const QModelIndex &index)
 
         drive->eject();
     } else {
-        QString label = data(index, Qt::DisplayRole).toString().replace('&', "&&");
+        QString label = data(index, Qt::DisplayRole).toString().replace('&', QLatin1String("&&"));
         QString message = i18n("The device '%1' is not a disk and cannot be ejected.", label);
         emit errorMessage(message);
     }
