@@ -26,6 +26,7 @@
 #include <kfileitem.h>
 #include <kiconeffect.h>
 #include <kio/previewjob.h>
+#include <kio/paste.h>
 #include <kdirlister.h>
 #include <kdirmodel.h>
 #include <kiconloader.h>
@@ -286,12 +287,6 @@ public:
      */
     void orderItems(KFileItemList &items);
 
-    /**
-     * Returns true, if \a mimeData represents a selection that has
-     * been cut.
-     */
-    bool decodeIsCutSelection(const QMimeData *mimeData);
-
     /**
      * Helper method for KFilePreviewGenerator::updateIcons(). Adds
      * recursively all items from the model to the list \a list.
@@ -482,6 +477,8 @@ KFilePreviewGenerator::Private::Private(KFilePreviewGenerator *parent,
     m_scrollAreaTimer->setInterval(200);
     connect(m_scrollAreaTimer, SIGNAL(timeout()),
             q, SLOT(resumeIconUpdates()));
+    m_viewAdapter->connect(KAbstractViewAdapter::IconSizeChanged,
+                           q, SLOT(updateIcons()));
     m_viewAdapter->connect(KAbstractViewAdapter::ScrollBarValueChanged,
                            q, SLOT(pauseIconUpdates()));
 
@@ -493,15 +490,15 @@ KFilePreviewGenerator::Private::Private(KFilePreviewGenerator *parent,
 
     KConfigGroup globalConfig(KSharedConfig::openConfig(), "PreviewSettings");
     m_enabledPlugins = globalConfig.readEntry("Plugins", QStringList()
-                       << "directorythumbnail"
-                       << "imagethumbnail"
-                       << "jpegthumbnail");
+                       << QStringLiteral("directorythumbnail")
+                       << QStringLiteral("imagethumbnail")
+                       << QStringLiteral("jpegthumbnail"));
 
-    // If the user is upgrading from KDE <= 4.6, we must check if he had the 'jpegrotatedthumbnail' plugin enabled.
-    // This plugin does not exist any more in KDE >= 4.7, so we have to replace it with the 'jpegthumbnail' plugin.
-    if (m_enabledPlugins.contains(QLatin1String("jpegrotatedthumbnail"))) {
-        m_enabledPlugins.removeAll(QLatin1String("jpegrotatedthumbnail"));
-        m_enabledPlugins.append(QLatin1String("jpegthumbnail"));
+    // Compatibility update: in 4.7, jpegrotatedthumbnail was merged into (or
+    // replaced with?) jpegthumbnail
+    if (m_enabledPlugins.contains(QStringLiteral("jpegrotatedthumbnail"))) {
+        m_enabledPlugins.removeAll(QStringLiteral("jpegrotatedthumbnail"));
+        m_enabledPlugins.append(QStringLiteral("jpegthumbnail"));
         globalConfig.writeEntry("Plugins", m_enabledPlugins);
         globalConfig.sync();
     }
@@ -673,6 +670,8 @@ void KFilePreviewGenerator::Private::addToPreviewQueue(const KFileItem &item, co
     preview.pixmap = icon;
     m_previews.append(preview);
 
+    m_pendingItems.removeOne(item);
+
     m_dispatchedItems.append(item);
 }
 
@@ -682,6 +681,12 @@ void KFilePreviewGenerator::Private::slotPreviewJobFinished(KJob *job)
     m_previewJobs.removeAt(index);
 
     if (m_previewJobs.isEmpty()) {
+        foreach (const KFileItem &item, m_pendingItems) {
+            if (item.isMimeTypeKnown()) {
+                m_resolvedMimeTypes.append(item);
+            }
+        }
+
         if (m_clearItemQueues) {
             m_pendingItems.clear();
             m_dispatchedItems.clear();
@@ -748,8 +753,7 @@ void KFilePreviewGenerator::Private::dispatchIconUpdateQueue()
         return;
     }
 
-    const int count = m_previewShown ? m_previews.count()
-                      : m_resolvedMimeTypes.count();
+    const int count = m_previews.count() + m_resolvedMimeTypes.count();
     if (count > 0) {
         LayoutBlocker blocker(m_itemView);
         DataChangeObtainer obt(this);
@@ -763,15 +767,15 @@ void KFilePreviewGenerator::Private::dispatchIconUpdateQueue()
                 }
             }
             m_previews.clear();
-        } else {
-            // dispatch mime type queue
-            foreach (const KFileItem &item, m_resolvedMimeTypes) {
-                const QModelIndex idx = dirModel->indexForItem(item);
-                dirModel->itemChanged(idx);
-            }
-            m_resolvedMimeTypes.clear();
         }
 
+        // dispatch mime type queue
+        foreach (const KFileItem &item, m_resolvedMimeTypes) {
+            const QModelIndex idx = dirModel->indexForItem(item);
+            dirModel->itemChanged(idx);
+        }
+        m_resolvedMimeTypes.clear();
+
         m_pendingVisibleIconUpdates -= count;
         if (m_pendingVisibleIconUpdates < 0) {
             m_pendingVisibleIconUpdates = 0;
@@ -896,7 +900,7 @@ bool KFilePreviewGenerator::Private::isCutItem(const KFileItem &item) const
 void KFilePreviewGenerator::Private::applyCutItemEffect(const KFileItemList &items)
 {
     const QMimeData *mimeData = QApplication::clipboard()->mimeData();
-    m_hasCutSelection = decodeIsCutSelection(mimeData);
+    m_hasCutSelection = mimeData && KIO::isClipboardDataCut(mimeData);
     if (!m_hasCutSelection) {
         return;
     }
@@ -977,7 +981,7 @@ bool KFilePreviewGenerator::Private::applyImageFrame(QPixmap &icon)
 void KFilePreviewGenerator::Private::limitToSize(QPixmap &icon, const QSize &maxSize)
 {
     if ((icon.width() > maxSize.width()) || (icon.height() > maxSize.height())) {
-#warning Cannot use XRender with QPixmap anymore. Find equivalent with Qt API.
+#pragma message("Cannot use XRender with QPixmap anymore. Find equivalent with Qt API.")
 #if 0 // HAVE_X11 && HAVE_XRENDER
         // Assume that the texture size limit is 2048x2048
         if ((icon.width() <= 2048) && (icon.height() <= 2048) && icon.x11PictureHandle()) {
@@ -1023,7 +1027,7 @@ void KFilePreviewGenerator::Private::createPreviews(const KFileItemList &items)
     }
 
     const QMimeData *mimeData = QApplication::clipboard()->mimeData();
-    m_hasCutSelection = decodeIsCutSelection(mimeData);
+    m_hasCutSelection = mimeData && KIO::isClipboardDataCut(mimeData);
 
     // PreviewJob internally caches items always with the size of
     // 128 x 128 pixels or 256 x 256 pixels. A downscaling is done
@@ -1126,16 +1130,6 @@ void KFilePreviewGenerator::Private::orderItems(KFileItemList &items)
     }
 }
 
-bool KFilePreviewGenerator::Private::decodeIsCutSelection(const QMimeData *mimeData)
-{
-    const QByteArray data = mimeData->data("application/x-kde-cutselection");
-    if (data.isEmpty()) {
-        return false;
-    } else {
-        return data.at(0) == QLatin1Char('1');
-    }
-}
-
 void KFilePreviewGenerator::Private::addItemsToList(const QModelIndex &index, KFileItemList &list)
 {
     KDirModel *dirModel = m_dirModel.data();
