@@ -29,7 +29,9 @@
 #include <QtCore/QObject>
 #include <QtCore/QHash>
 #include <QtCore/QFile>
-#include<qplatformdefs.h>
+#include <KUser>
+
+#include <qplatformdefs.h> // mode_t
 #include <config-kioslave-file.h>
 
 #if HAVE_POSIX_ACL
@@ -44,35 +46,35 @@ public:
     FileProtocol(const QByteArray &pool, const QByteArray &app);
     virtual ~FileProtocol();
 
-    virtual void get(const QUrl &url);
+    void get(const QUrl &url) Q_DECL_OVERRIDE;
     virtual void put(const QUrl &url, int _mode,
-                     KIO::JobFlags _flags);
+                     KIO::JobFlags _flags) Q_DECL_OVERRIDE;
     virtual void copy(const QUrl &src, const QUrl &dest,
-                      int mode, KIO::JobFlags flags);
+                      int mode, KIO::JobFlags flags) Q_DECL_OVERRIDE;
     virtual void rename(const QUrl &src, const QUrl &dest,
-                        KIO::JobFlags flags);
+                        KIO::JobFlags flags) Q_DECL_OVERRIDE;
     virtual void symlink(const QString &target, const QUrl &dest,
-                         KIO::JobFlags flags);
+                         KIO::JobFlags flags) Q_DECL_OVERRIDE;
 
-    virtual void stat(const QUrl &url);
-    virtual void listDir(const QUrl &url);
-    virtual void mkdir(const QUrl &url, int permissions);
-    virtual void chmod(const QUrl &url, int permissions);
-    virtual void chown(const QUrl &url, const QString &owner, const QString &group);
-    virtual void setModificationTime(const QUrl &url, const QDateTime &mtime);
-    virtual void del(const QUrl &url, bool isfile);
-    virtual void open(const QUrl &url, QIODevice::OpenMode mode);
-    virtual void read(KIO::filesize_t size);
-    virtual void write(const QByteArray &data);
-    virtual void seek(KIO::filesize_t offset);
-    virtual void close();
+    void stat(const QUrl &url) Q_DECL_OVERRIDE;
+    void listDir(const QUrl &url) Q_DECL_OVERRIDE;
+    void mkdir(const QUrl &url, int permissions) Q_DECL_OVERRIDE;
+    void chmod(const QUrl &url, int permissions) Q_DECL_OVERRIDE;
+    void chown(const QUrl &url, const QString &owner, const QString &group) Q_DECL_OVERRIDE;
+    void setModificationTime(const QUrl &url, const QDateTime &mtime) Q_DECL_OVERRIDE;
+    void del(const QUrl &url, bool isfile) Q_DECL_OVERRIDE;
+    void open(const QUrl &url, QIODevice::OpenMode mode) Q_DECL_OVERRIDE;
+    void read(KIO::filesize_t size) Q_DECL_OVERRIDE;
+    void write(const QByteArray &data) Q_DECL_OVERRIDE;
+    void seek(KIO::filesize_t offset) Q_DECL_OVERRIDE;
+    void close() Q_DECL_OVERRIDE;
 
     /**
      * Special commands supported by this slave:
      * 1 - mount
      * 2 - unmount
      */
-    virtual void special(const QByteArray &data);
+    void special(const QByteArray &data) Q_DECL_OVERRIDE;
     void unmount(const QString &point);
     void mount(bool _ro, const char *_fstype, const QString &dev, const QString &point);
     bool pumount(const QString &point);
@@ -82,19 +84,22 @@ public:
     static bool isExtendedACL(acl_t acl);
 #endif
 
+protected:
+    void virtual_hook(int id, void *data) Q_DECL_OVERRIDE;
+
 private:
     bool createUDSEntry(const QString &filename, const QByteArray &path, KIO::UDSEntry &entry,
-                        short int details, bool withACL);
+                        short int details);
     int setACL(const char *path, mode_t perm, bool _directoryDefault);
-
-    QString getUserName(uid_t uid) const;
-    QString getGroupName(gid_t gid) const;
-
+    QString getUserName(KUserId uid) const;
+    QString getGroupName(KGroupId gid) const;
     bool deleteRecursive(const QString &path);
 
+    void fileSystemFreeSpace(const QUrl &url);  // KF6 TODO: Turn into virtual method in SlaveBase
+
 private:
-    mutable QHash<uid_t, QString> mUsercache;
-    mutable QHash<gid_t, QString> mGroupcache;
+    mutable QHash<KUserId, QString> mUsercache;
+    mutable QHash<KGroupId, QString> mGroupcache;
     QFile *mFile;
 };
 
