@@ -273,7 +273,7 @@ void FileProtocol::copy(const QUrl &srcUrl, const QUrl &destUrl,
     ut.actime = buff_src.st_atime;
     ut.modtime = buff_src.st_mtime;
     if (::utime(_dest.data(), &ut) != 0) {
-        qWarning() << QString::fromLatin1("Couldn't preserve access and modification time for\n%1").arg(dest);
+        qWarning() << QStringLiteral("Couldn't preserve access and modification time for\n%1").arg(dest);
     }
 
     processedSize(buff_src.st_size);
@@ -347,7 +347,7 @@ void FileProtocol::listDir(const QUrl &url)
         return;
     }
 
-    const QString sDetails = metaData(QLatin1String("details"));
+    const QString sDetails = metaData(QStringLiteral("details"));
     const int details = sDetails.isEmpty() ? 2 : sDetails.toInt();
     //qDebug() << "========= LIST " << url << "details=" << details << " =========";
     UDSEntry entry;
@@ -389,7 +389,7 @@ void FileProtocol::listDir(const QUrl &url)
             if (isSymLink) {
                 // for symlinks obey the UDSEntry contract and provide UDS_LINK_DEST
                 // even if we don't know the link dest (and DeleteJob doesn't care...)
-                entry.insert(KIO::UDSEntry::UDS_LINK_DEST, QLatin1String("Dummy Link Target"));
+                entry.insert(KIO::UDSEntry::UDS_LINK_DEST, QStringLiteral("Dummy Link Target"));
             }
             listEntry(entry);
 
@@ -451,7 +451,7 @@ void FileProtocol::rename(const QUrl &srcUrl, const QUrl &destUrl,
         if ((errno == EACCES) || (errno == EPERM)) {
             error(KIO::ERR_ACCESS_DENIED, dest);
         } else if (errno == EXDEV) {
-            error(KIO::ERR_UNSUPPORTED_ACTION, QLatin1String("rename"));
+            error(KIO::ERR_UNSUPPORTED_ACTION, QStringLiteral("rename"));
         } else if (errno == EROFS) { // The file is on a read-only filesystem
             error(KIO::ERR_CANNOT_DELETE, src);
         } else {
@@ -478,6 +478,7 @@ void FileProtocol::symlink(const QString &target, const QUrl &destUrl, KIO::JobF
                 }
                 // Try again - this won't loop forever since unlink succeeded
                 symlink(target, destUrl, flags);
+                return;
             } else {
                 QT_STATBUF buff_dest;
                 if (QT_LSTAT(QFile::encodeName(dest).constData(), &buff_dest) == 0 && ((buff_dest.st_mode & QT_STAT_MASK) == QT_STAT_DIR)) {
@@ -524,7 +525,7 @@ void FileProtocol::del(const QUrl &url, bool isfile)
          *****/
 
         // qDebug() << "Deleting directory " << url;
-        if (metaData(QLatin1String("recurse")) == QLatin1String("true")) {
+        if (metaData(QStringLiteral("recurse")) == QLatin1String("true")) {
             if (!deleteRecursive(path)) {
                 return;
             }
@@ -613,7 +614,7 @@ void FileProtocol::stat(const QUrl &url)
      */
     const QString path(url.adjusted(QUrl::StripTrailingSlash).toLocalFile());
     const QByteArray _path(QFile::encodeName(path));
-    const QString sDetails = metaData(QLatin1String("details"));
+    const QString sDetails = metaData(QStringLiteral("details"));
     const int details = sDetails.isEmpty() ? 2 : sDetails.toInt();
 
     UDSEntry entry;
