@@ -131,8 +131,8 @@ void KCookiesManagement::save()
   // If delete all cookies was requested!
   if(mDeleteAllFlag)
   {
-    QDBusInterface kded("org.kde.kded", "/modules/kcookiejar", "org.kde.KCookieServer", QDBusConnection::sessionBus());
-    QDBusReply<void> reply = kded.call( "deleteAllCookies" );
+    QDBusInterface kded(QStringLiteral("org.kde.kded5"), QStringLiteral("/modules/kcookiejar"), QStringLiteral("org.kde.KCookieServer"), QDBusConnection::sessionBus());
+    QDBusReply<void> reply = kded.call( QStringLiteral("deleteAllCookies") );
     if (!reply.isValid())
     {
       QString caption = i18n ("D-Bus Communication Error");
@@ -147,8 +147,8 @@ void KCookiesManagement::save()
   QMutableStringListIterator it (mDeletedDomains);
   while (it.hasNext())
   {    
-    QDBusInterface kded("org.kde.kded", "/modules/kcookiejar", "org.kde.KCookieServer", QDBusConnection::sessionBus());
-    QDBusReply<void> reply = kded.call( "deleteCookiesFromDomain",( it.next() ) );
+    QDBusInterface kded(QStringLiteral("org.kde.kded5"), QStringLiteral("/modules/kcookiejar"), QStringLiteral("org.kde.KCookieServer"), QDBusConnection::sessionBus());
+    QDBusReply<void> reply = kded.call( QStringLiteral("deleteCookiesFromDomain"),( it.next() ) );
     if (!reply.isValid())
     {
       QString caption = i18n ("D-Bus Communication Error");
@@ -168,8 +168,8 @@ void KCookiesManagement::save()
     CookiePropList list = cookiesDom.value();
     foreach(CookieProp *cookie, list)
     {
-      QDBusInterface kded("org.kde.kded", "/modules/kcookiejar", "org.kde.KCookieServer", QDBusConnection::sessionBus());
-      QDBusReply<void> reply = kded.call( "deleteCookie", cookie->domain,
+      QDBusInterface kded(QStringLiteral("org.kde.kded5"), QStringLiteral("/modules/kcookiejar"), QStringLiteral("org.kde.KCookieServer"), QDBusConnection::sessionBus());
+      QDBusReply<void> reply = kded.call( QStringLiteral("deleteCookie"), cookie->domain,
                                           cookie->host, cookie->path,
                                           cookie->name );
       if (!reply.isValid())
@@ -228,8 +228,8 @@ QString KCookiesManagement::quickHelp() const
 
 void KCookiesManagement::on_reloadButton_clicked()
 {
-  QDBusInterface kded("org.kde.kded", "/modules/kcookiejar", "org.kde.KCookieServer", QDBusConnection::sessionBus());
-  QDBusReply<QStringList> reply = kded.call( "findDomains" );
+  QDBusInterface kded(QStringLiteral("org.kde.kded5"), QStringLiteral("/modules/kcookiejar"), QStringLiteral("org.kde.KCookieServer"), QDBusConnection::sessionBus());
+  QDBusReply<QStringList> reply = kded.call( QStringLiteral("findDomains") );
 
   if (!reply.isValid())
   {
@@ -273,8 +273,8 @@ void KCookiesManagement::on_cookiesTreeWidget_itemExpanded(QTreeWidgetItem *item
   fields << 0 << 1 << 2 << 3;  
   // Always check for cookies in both "foo.bar" and ".foo.bar" domains...
   const QString domain = cookieDom->domain() + QLatin1String(" .") + cookieDom->domain();
-  QDBusInterface kded("org.kde.kded", "/modules/kcookiejar", "org.kde.KCookieServer", QDBusConnection::sessionBus());  
-  QDBusReply<QStringList> reply = kded.call("findCookies", QVariant::fromValue(fields),
+  QDBusInterface kded(QStringLiteral("org.kde.kded5"), QStringLiteral("/modules/kcookiejar"), QStringLiteral("org.kde.KCookieServer"), QDBusConnection::sessionBus());
+  QDBusReply<QStringList> reply = kded.call(QStringLiteral("findCookies"), QVariant::fromValue(fields),
                                             domain, QString(), QString(), QString());
   if (reply.isValid())
       cookies.append(reply.value());
@@ -303,8 +303,8 @@ bool KCookiesManagement::cookieDetails(CookieProp *cookie)
   QList<int> fields;
   fields << 4 << 5 << 7;
 
-  QDBusInterface kded("org.kde.kded", "/modules/kcookiejar", "org.kde.KCookieServer", QDBusConnection::sessionBus());
-  QDBusReply<QStringList> reply = kded.call( "findCookies",
+  QDBusInterface kded(QStringLiteral("org.kde.kded5"), QStringLiteral("/modules/kcookiejar"), QStringLiteral("org.kde.KCookieServer"), QDBusConnection::sessionBus());
+  QDBusReply<QStringList> reply = kded.call( QStringLiteral("findCookies"),
                                              QVariant::fromValue( fields ),
                                              cookie->domain,
                                              cookie->host,
