@@ -51,9 +51,9 @@
 #define QL1C(x)   QLatin1Char(x)
 
 #if QT_VERSION >= 0x040800
-static QNetworkRequest::Attribute gSynchronousNetworkRequestAttribute = QNetworkRequest::SynchronousRequestAttribute;
+static const QNetworkRequest::Attribute gSynchronousNetworkRequestAttribute = QNetworkRequest::SynchronousRequestAttribute;
 #else // QtWebkit hack to use the internal attribute
-static QNetworkRequest::Attribute gSynchronousNetworkRequestAttribute = static_cast<QNetworkRequest::Attribute>(QNetworkRequest::HttpPipeliningWasUsedAttribute + 7);
+static const QNetworkRequest::Attribute gSynchronousNetworkRequestAttribute = static_cast<QNetworkRequest::Attribute>(QNetworkRequest::HttpPipeliningWasUsedAttribute + 7);
 #endif
 
 static qint64 sizeFromRequest(const QNetworkRequest &req)
@@ -133,7 +133,7 @@ bool AccessManager::isExternalContentAllowed() const
     return d->externalContentAllowed;
 }
 
-#ifndef KDE_NO_DEPRECATED
+#ifndef KIOWIDGETS_NO_DEPRECATED
 void AccessManager::setCookieJarWindowId(WId id)
 {
     QWidget *window = QWidget::find(id);
@@ -168,7 +168,7 @@ void AccessManager::setWindow(QWidget *widget)
     }
 }
 
-#ifndef KDE_NO_DEPRECATED
+#ifndef KIOWIDGETS_NO_DEPRECATED
 WId AccessManager::cookieJarWindowid() const
 {
     KIO::Integration::CookieJar *jar = qobject_cast<KIO::Integration::CookieJar *> (cookieJar());
@@ -252,7 +252,16 @@ QNetworkReply *AccessManager::createRequest(Operation op, const QNetworkRequest
     case PutOperation: {
         //qDebug() << "PutOperation:" << reqUrl;
         if (outgoingData) {
-            kioJob = KIO::storedPut(outgoingData->readAll(), reqUrl, -1, KIO::HideProgressInfo);
+            Q_ASSERT(outgoingData->isReadable());
+            StoredTransferJob* storedJob = KIO::storedPut(outgoingData, reqUrl, -1, KIO::HideProgressInfo);
+            storedJob->setAsyncDataEnabled(outgoingData->isSequential());
+
+            QVariant len = req.header(QNetworkRequest::ContentLengthHeader);
+            if (len.isValid()) {
+                storedJob->setTotalSize(len.toInt());
+            }
+
+            kioJob = storedJob;
         } else {
             kioJob = KIO::put(reqUrl, -1, KIO::HideProgressInfo);
         }
@@ -343,7 +352,7 @@ QNetworkReply *AccessManager::createRequest(Operation op, const QNetworkRequest
             reply = new KDEPrivate::AccessManagerReply(op, req, QNetworkReply::UnknownNetworkError, kioJob->errorText(), this);
         }
     } else {
-        // Set the window on the the KIO ui delegate
+        // Set the window on the KIO ui delegate
         if (d->window) {
             KJobWidgets::setWindow(kioJob, d->window);
         }
@@ -438,7 +447,7 @@ void AccessManager::AccessManagerPrivate::setMetaDataForRequest(QNetworkRequest
     }
 
     if (!customHeaders.isEmpty()) {
-        metaData.insert(QL1S("customHTTPHeader"), customHeaders.join("\r\n"));
+        metaData.insert(QL1S("customHTTPHeader"), customHeaders.join(QStringLiteral("\r\n")));
     }
 
     // Append per request meta data, if any...
@@ -518,8 +527,8 @@ QList<QNetworkCookie> CookieJar::cookiesForUrl(const QUrl &url) const
     if (!d->isEnabled) {
         return cookieList;
     }
-    QDBusInterface kcookiejar("org.kde.kded5", "/modules/kcookiejar", "org.kde.KCookieServer");
-    QDBusReply<QString> reply = kcookiejar.call("findDOMCookies", url.toString(QUrl::RemoveUserInfo), (qlonglong)d->windowId);
+    QDBusInterface kcookiejar(QStringLiteral("org.kde.kded5"), QStringLiteral("/modules/kcookiejar"), QStringLiteral("org.kde.KCookieServer"));
+    QDBusReply<QString> reply = kcookiejar.call(QStringLiteral("findDOMCookies"), url.toString(QUrl::RemoveUserInfo), (qlonglong)d->windowId);
 
     if (!reply.isValid()) {
         qWarning() << "Unable to communicate with the cookiejar!";
@@ -545,7 +554,7 @@ bool CookieJar::setCookiesFromUrl(const QList<QNetworkCookie> &cookieList, const
         return false;
     }
 
-    QDBusInterface kcookiejar("org.kde.kded5", "/modules/kcookiejar", "org.kde.KCookieServer");
+    QDBusInterface kcookiejar(QStringLiteral("org.kde.kded5"), QStringLiteral("/modules/kcookiejar"), QStringLiteral("org.kde.KCookieServer"));
     Q_FOREACH (const QNetworkCookie &cookie, cookieList) {
         QByteArray cookieHeader("Set-Cookie: ");
         if (d->isStorageDisabled && !cookie.isSessionCookie()) {
@@ -555,7 +564,7 @@ bool CookieJar::setCookiesFromUrl(const QList<QNetworkCookie> &cookieList, const
         } else {
             cookieHeader += cookie.toRawForm();
         }
-        kcookiejar.call("addCookies", url.toString(QUrl::RemoveUserInfo), cookieHeader, (qlonglong)d->windowId);
+        kcookiejar.call(QStringLiteral("addCookies"), url.toString(QUrl::RemoveUserInfo), cookieHeader, (qlonglong)d->windowId);
         //qDebug() << "[" << d->windowId << "]" << cookieHeader << " from " << url;
     }
 
@@ -574,7 +583,7 @@ void CookieJar::setWindowId(WId id)
 
 void CookieJar::reparseConfiguration()
 {
-    KConfigGroup cfg = KSharedConfig::openConfig("kcookiejarrc", KConfig::NoGlobals)->group("Cookie Policy");
+    KConfigGroup cfg = KSharedConfig::openConfig(QStringLiteral("kcookiejarrc"), KConfig::NoGlobals)->group("Cookie Policy");
     d->isEnabled = cfg.readEntry("Cookies", true);
 }
 
