@@ -40,6 +40,8 @@
 #include <QDesktopWidget>
 #include <qmimedatabase.h>
 #include <QDebug>
+#include <QDBusConnection>
+#include <QDBusConnectionInterface>
 
 #include <kiconloader.h>
 #include <kjobuidelegate.h>
@@ -112,7 +114,7 @@ static QString schemeHandler(const QString &protocol)
     //     to a webbrowser, but we want mimetype-determination-in-calling-application by default
     //     (the user can configure a BrowserApplication though)
 
-    const KService::Ptr service = KMimeTypeTrader::self()->preferredService(QString::fromLatin1("x-scheme-handler/") + protocol);
+    const KService::Ptr service = KMimeTypeTrader::self()->preferredService(QLatin1String("x-scheme-handler/") + protocol);
     if (service) {
         return service->exec(); // for helper protocols, the handler app has priority over the hardcoded one (see A above)
     }
@@ -131,11 +133,11 @@ bool KRun::isExecutableFile(const QUrl &url, const QString &mimetype)
     if (file.isExecutable()) {    // Got a prospective file to run
         QMimeDatabase db;
         QMimeType mimeType = db.mimeTypeForName(mimetype);
-        if (mimeType.inherits(QLatin1String("application/x-executable")) ||
+        if (mimeType.inherits(QStringLiteral("application/x-executable")) ||
 #ifdef Q_OS_WIN
                 mimeType.inherits(QLatin1String("application/x-ms-dos-executable")) ||
 #endif
-                mimeType.inherits(QLatin1String("application/x-executable-script"))
+                mimeType.inherits(QStringLiteral("application/x-executable-script"))
            ) {
             return true;
         }
@@ -172,11 +174,11 @@ bool KRun::runUrl(const QUrl &u, const QString &_mimetype, QWidget *window, bool
         return false;
     } else if (_mimetype == QLatin1String("application/x-desktop")) {
         if (u.isLocalFile() && runExecutables) {
-            return KDesktopFileActions::run(u, true);
+            return KDesktopFileActions::runWithStartup(u, true, asn);
         }
     } else if (isExecutableFile(u, _mimetype)) {
         if (u.isLocalFile() && runExecutables) {
-            if (KAuthorized::authorize("shell_access")) {
+            if (KAuthorized::authorize(QStringLiteral("shell_access"))) {
                 return (KRun::runCommand(KShell::quoteArg(u.toLocalFile()), QString(), QString(),
                             window, asn, u.adjusted(QUrl::RemoveFilename).toLocalFile())); // just execute the url as a command
                 // ## TODO implement deleting the file if tempFile==true
@@ -191,7 +193,7 @@ bool KRun::runUrl(const QUrl &u, const QString &_mimetype, QWidget *window, bool
             noRun = true;
         }
 
-        if (!KAuthorized::authorize("shell_access")) {
+        if (!KAuthorized::authorize(QStringLiteral("shell_access"))) {
             noAuth = true;
         }
     }
@@ -226,13 +228,13 @@ bool KRun::runUrl(const QUrl &u, const QString &_mimetype, QWidget *window, bool
 #endif
     }
 
-    return KRun::run(*offer, lst, window, tempFile, suggestedFileName, asn);
+    return KRun::runService(*offer, lst, window, tempFile, suggestedFileName, asn);
 }
 
 bool KRun::displayOpenWithDialog(const QList<QUrl> &lst, QWidget *window, bool tempFiles,
                                  const QString &suggestedFileName, const QByteArray &asn)
 {
-    if (!KAuthorized::authorizeKAction("openwith")) {
+    if (!KAuthorized::authorizeKAction(QStringLiteral("openwith"))) {
         KMessageBox::sorry(window,
                            i18n("You are not authorized to select an application to open this file."));
         return false;
@@ -253,7 +255,7 @@ bool KRun::displayOpenWithDialog(const QList<QUrl> &lst, QWidget *window, bool t
             //qDebug() << "No service set, running " << l.text();
             service = KService::Ptr(new KService(QString() /*name*/, l.text(), QString() /*icon*/));
         }
-        return KRun::run(*service, lst, window, tempFiles, suggestedFileName, asn);
+        return KRun::runService(*service, lst, window, tempFiles, suggestedFileName, asn);
     }
     return false;
 }
@@ -266,7 +268,7 @@ void KRun::shellQuote(QString &_str)
         return;
     }
     QChar q('\'');
-    _str.replace(q, "'\\''").prepend(q).append(q);
+    _str.replace(q, QLatin1String("'\\''")).prepend(q).append(q);
 }
 #endif
 
@@ -285,18 +287,19 @@ QString KRun::binaryName(const QString &execLine, bool removePath)
 }
 #endif
 
-static bool runCommandInternal(const QString &command, const KService *service, const QString &executable,
+static qint64 runCommandInternal(KProcess *proc, const KService *service, const QString &executable,
                                const QString &userVisibleName, const QString &iconName, QWidget *window,
-                               const QByteArray &asn, QString workingDirectory = QString())
+                               const QByteArray &asn)
 {
-    if (window != NULL) {
+    if (window) {
         window = window->topLevelWidget();
     }
     if (service && !service->entryPath().isEmpty()
             && !KDesktopFile::isAuthorizedDesktopFile(service->entryPath())) {
         qWarning() << "No authorization to execute " << service->entryPath();
         KMessageBox::sorry(window, i18n("You are not authorized to execute this file."));
-        return false;
+        delete proc;
+        return 0;
     }
 
     QString bin = KIO::DesktopExecParser::executableName(executable);
@@ -339,20 +342,20 @@ static bool runCommandInternal(const QString &command, const KService *service,
             }
             KStartupInfo::sendStartup(id, data);
         }
-        int pid = KProcessRunner::run(command, executable, id, workingDirectory);
+        qint64 pid = KProcessRunner::run(proc, executable, id);
         if (startup_notify && pid) {
             KStartupInfoData data;
             data.addPid(pid);
             KStartupInfo::sendChange(id, data);
             KStartupInfo::resetStartupEnv();
         }
-        return pid != 0;
+        return pid;
     }
 #else
     Q_UNUSED(userVisibleName);
     Q_UNUSED(iconName);
 #endif
-    return KProcessRunner::run(command, bin, KStartupInfoId(), workingDirectory) != 0;
+    return KProcessRunner::run(proc, bin, KStartupInfoId());
 }
 
 // This code is also used in klauncher.
@@ -360,12 +363,12 @@ bool KRun::checkStartupNotify(const QString & /*binName*/, const KService *servi
 {
     bool silent = false;
     QByteArray wmclass;
-    if (service && service->property("StartupNotify").isValid()) {
-        silent = !service->property("StartupNotify").toBool();
-        wmclass = service->property("StartupWMClass").toString().toLatin1();
-    } else if (service && service->property("X-KDE-StartupNotify").isValid()) {
-        silent = !service->property("X-KDE-StartupNotify").toBool();
-        wmclass = service->property("X-KDE-WMClass").toString().toLatin1();
+    if (service && service->property(QStringLiteral("StartupNotify")).isValid()) {
+        silent = !service->property(QStringLiteral("StartupNotify")).toBool();
+        wmclass = service->property(QStringLiteral("StartupWMClass")).toString().toLatin1();
+    } else if (service && service->property(QStringLiteral("X-KDE-StartupNotify")).isValid()) {
+        silent = !service->property(QStringLiteral("X-KDE-StartupNotify")).toBool();
+        wmclass = service->property(QStringLiteral("X-KDE-WMClass")).toString().toLatin1();
     } else { // non-compliant app
         if (service) {
             if (service->isApplication()) { // doesn't have .desktop entries needed, start as non-compliant
@@ -386,21 +389,19 @@ bool KRun::checkStartupNotify(const QString & /*binName*/, const KService *servi
 #endif
         }
     }
-    if (silent_arg != NULL) {
+    if (silent_arg) {
         *silent_arg = silent;
     }
-    if (wmclass_arg != NULL) {
+    if (wmclass_arg) {
         *wmclass_arg = wmclass;
     }
     return true;
 }
 
-static bool runTempService(const KService &_service, const QList<QUrl> &_urls, QWidget *window,
+static qint64 runTempService(const KService &_service, const QList<QUrl> &_urls, QWidget *window,
                            bool tempFiles, const QString &suggestedFileName, const QByteArray &asn)
 {
-    if (!_urls.isEmpty()) {
-        //qDebug() << "runTempService: first url " << _urls.first();
-    }
+    //qDebug() << "runTempService:" << _urls;
 
     QList<QUrl> urlsToRun = _urls;
     if ((_urls.count() > 1) && !_service.allowMultipleFiles()) {
@@ -424,17 +425,22 @@ static bool runTempService(const KService &_service, const QList<QUrl> &_urls, Q
     const QStringList args = execParser.resultingArguments();
     if (args.isEmpty()) {
         KMessageBox::sorry(window, i18n("Error processing Exec field in %1", _service.entryPath()));
-        return false;
+        return 0;
     }
     //qDebug() << "runTempService: KProcess args=" << args;
 
+    KProcess * proc = new KProcess;
+    *proc << args;
+
     QString path(_service.path());
     if (path.isEmpty() && !_urls.isEmpty() && _urls.first().isLocalFile()) {
         path = _urls.first().adjusted(QUrl::RemoveFilename).toLocalFile();
     }
 
-    return runCommandInternal(args.join(" "), &_service, KIO::DesktopExecParser::executablePath(_service.exec()),
-                              _service.name(), _service.icon(), window, asn, path);
+    proc->setWorkingDirectory(path);
+
+    return runCommandInternal(proc, &_service, KIO::DesktopExecParser::executablePath(_service.exec()),
+                              _service.name(), _service.icon(), window, asn);
 }
 
 // WARNING: don't call this from DesktopExecParser, since klauncher uses that too...
@@ -445,12 +451,12 @@ static QList<QUrl> resolveURLs(const QList<QUrl> &_urls, const KService &_servic
     // This can be a list of actual protocol names, or just KIO for KDE apps.
     QStringList appSupportedProtocols = KIO::DesktopExecParser::supportedProtocols(_service);
     QList<QUrl> urls(_urls);
-    if (!appSupportedProtocols.contains("KIO")) {
+    if (!appSupportedProtocols.contains(QStringLiteral("KIO"))) {
         for (QList<QUrl>::Iterator it = urls.begin(); it != urls.end(); ++it) {
             const QUrl url = *it;
             bool supported = KIO::DesktopExecParser::isProtocolInSupportedList(url, appSupportedProtocols);
             //qDebug() << "Looking at url=" << url << " supported=" << supported;
-            if (!supported && KProtocolInfo::protocolClass(url.scheme()) == ":local") {
+            if (!supported && KProtocolInfo::protocolClass(url.scheme()) == QLatin1String(":local")) {
                 // Maybe we can resolve to a local URL?
                 KIO::StatJob *job = KIO::mostLocalUrl(url);
                 if (job->exec()) { // ## nasty nested event loop!
@@ -481,7 +487,7 @@ public:
     }
 
 protected:
-    virtual void showEvent(QShowEvent *e)
+    void showEvent(QShowEvent *e) Q_DECL_OVERRIDE
     {
         // Now that we're shown, use our width to calculate a good
         // bounding box for the text, and resize m_textEdit appropriately.
@@ -598,7 +604,7 @@ static bool makeFileExecutable(const QString &fileName)
 // to make the file executable or we failed to make it executable.
 static bool makeServiceExecutable(const KService &service, QWidget *window)
 {
-    if (!KAuthorized::authorize("run_desktop_files")) {
+    if (!KAuthorized::authorize(QStringLiteral("run_desktop_files"))) {
         qWarning() << "No authorization to execute " << service.entryPath();
         KMessageBox::sorry(window, i18n("You are not authorized to execute this service."));
         return false; // Don't circumvent the Kiosk
@@ -616,7 +622,7 @@ static bool makeServiceExecutable(const KService &service, QWidget *window)
     QHBoxLayout *mainLayout = new QHBoxLayout(baseWidget);
 
     QLabel *iconLabel = new QLabel(baseWidget);
-    QPixmap warningIcon(KIconLoader::global()->loadIcon("dialog-warning", KIconLoader::NoGroup, KIconLoader::SizeHuge));
+    QPixmap warningIcon(KIconLoader::global()->loadIcon(QStringLiteral("dialog-warning"), KIconLoader::NoGroup, KIconLoader::SizeHuge));
     mainLayout->addWidget(iconLabel);
     iconLabel->setPixmap(warningIcon);
 
@@ -693,11 +699,17 @@ static bool makeServiceExecutable(const KService &service, QWidget *window)
 
 bool KRun::run(const KService &_service, const QList<QUrl> &_urls, QWidget *window,
                bool tempFiles, const QString &suggestedFileName, const QByteArray &asn)
+{
+    return runService(_service, _urls, window, tempFiles, suggestedFileName, asn) != 0;
+}
+
+qint64 KRun::runService(const KService &_service, const QList<QUrl> &_urls, QWidget *window,
+                      bool tempFiles, const QString &suggestedFileName, const QByteArray &asn)
 {
     if (!_service.entryPath().isEmpty() &&
             !KDesktopFile::isAuthorizedDesktopFile(_service.entryPath()) &&
             !::makeServiceExecutable(_service, window)) {
-        return false;
+        return 0;
     }
 
     if (!tempFiles) {
@@ -709,25 +721,36 @@ bool KRun::run(const KService &_service, const QList<QUrl> &_urls, QWidget *wind
         }
     }
 
-    if (tempFiles || _service.entryPath().isEmpty() || !suggestedFileName.isEmpty()) {
-        return runTempService(_service, _urls, window, tempFiles, suggestedFileName, asn);
+    bool useKToolInvocation = !(tempFiles || _service.entryPath().isEmpty() || !suggestedFileName.isEmpty());
+
+    if (useKToolInvocation) {
+        // Is klauncher installed? Let's try to start it, if it fails, then we won't use it.
+        static int klauncherAvailable = -1;
+        if (klauncherAvailable == -1) {
+            KToolInvocation::ensureKdeinitRunning();
+            QDBusConnectionInterface *dbusDaemon = QDBusConnection::sessionBus().interface();
+            klauncherAvailable = dbusDaemon->isServiceRegistered(QStringLiteral("org.kde.klauncher5"));
+        }
+        if (klauncherAvailable == 0) {
+            useKToolInvocation = false;
+        }
     }
 
-    //qDebug() << "KRun::run " << _service.entryPath();
-
-    if (!_urls.isEmpty()) {
-        //qDebug() << "First url" << _urls.first();
+    if (!useKToolInvocation) {
+        return runTempService(_service, _urls, window, tempFiles, suggestedFileName, asn);
     }
 
     // Resolve urls if needed, depending on what the app supports
     const QList<QUrl> urls = resolveURLs(_urls, _service);
 
+    //qDebug() << "Running" << _service.entryPath() << _urls << "using klauncher";
+
     QString error;
-    int pid = 0; //TODO: change KToolInvokation to take a qint64*?
+    int pid = 0; //TODO KF6: change KToolInvokation to take a qint64*
 
     QByteArray myasn = asn;
     // startServiceByDesktopPath() doesn't take QWidget*, add it to the startup info now
-    if (window != NULL) {
+    if (window) {
         if (myasn.isEmpty()) {
             myasn = KStartupInfo::createNewStartupId();
         }
@@ -747,11 +770,11 @@ bool KRun::run(const KService &_service, const QList<QUrl> &_urls, QWidget *wind
     if (i != 0) {
         //qDebug() << error;
         KMessageBox::sorry(window, error);
-        return false;
+        return 0;
     }
 
     //qDebug() << "startServiceByDesktopPath worked fine";
-    return true;
+    return pid;
 }
 
 bool KRun::run(const QString &_exec, const QList<QUrl> &_urls, QWidget *window, const QString &_name,
@@ -759,7 +782,7 @@ bool KRun::run(const QString &_exec, const QList<QUrl> &_urls, QWidget *window,
 {
     KService::Ptr service(new KService(_name, _exec, _icon));
 
-    return run(*service, _urls, window, false, QString(), asn);
+    return runService(*service, _urls, window, false, QString(), asn);
 }
 
 bool KRun::runCommand(const QString &cmd, QWidget *window, const QString &workingDirectory)
@@ -795,10 +818,10 @@ bool KRun::runCommand(const QString &cmd, const QString &execName, const QString
     }
     QString bin = KIO::DesktopExecParser::executableName(execName);
     KService::Ptr service = KService::serviceByDesktopName(bin);
-    return runCommandInternal(cmd, service.data(),
+    return runCommandInternal(proc, service.data(),
                               execName /*executable to check for in slotProcessExited*/,
                               execName /*user-visible name*/,
-                              iconName, window, asn, workingDirectory);
+                              iconName, window, asn) != 0;
 }
 
 KRun::KRun(const QUrl &url, QWidget *window,
@@ -806,7 +829,7 @@ KRun::KRun(const QUrl &url, QWidget *window,
     : d(new KRunPrivate(this))
 {
     d->m_timer = new QTimer(this);
-    d->m_timer->setObjectName("KRun::timer");
+    d->m_timer->setObjectName(QStringLiteral("KRun::timer"));
     d->m_timer->setSingleShot(true);
     d->init(url, window, showProgressInfo, asn);
 }
@@ -848,7 +871,7 @@ void KRun::init()
         d->startTimer();
         return;
     }
-    if (!KUrlAuthorized::authorizeUrlAction("open", QUrl(), d->m_strURL)) {
+    if (!KUrlAuthorized::authorizeUrlAction(QStringLiteral("open"), QUrl(), d->m_strURL)) {
         QString msg = KIO::buildErrorString(KIO::ERR_ACCESS_DENIED, d->m_strURL.toDisplayString());
         handleInitError(KIO::ERR_ACCESS_DENIED, msg);
         d->m_bFault = true;
@@ -878,8 +901,8 @@ void KRun::init()
         QMimeType mime = db.mimeTypeForUrl(d->m_strURL);
         //qDebug() << "MIME TYPE is " << mime.name();
         if (!d->m_externalBrowser.isEmpty() && (
-                    mime.inherits(QLatin1String("text/html")) ||
-                    mime.inherits(QLatin1String("application/xhtml+xml")))) {
+                    mime.inherits(QStringLiteral("text/html")) ||
+                    mime.inherits(QStringLiteral("application/xhtml+xml")))) {
             if (d->runExecutable(d->m_externalBrowser)) {
                 return;
             }
@@ -954,7 +977,7 @@ bool KRun::KRunPrivate::runExecutable(const QString &_exec)
     urls.append(m_strURL);
     if (_exec.startsWith('!')) {
         QString exec = _exec.mid(1); // Literal command
-        exec += " %u";
+        exec += QLatin1String(" %u");
         if (q->run(exec, urls, m_window, QString(), QString(), m_asn)) {
             m_bFinished = true;
             startTimer();
@@ -962,7 +985,7 @@ bool KRun::KRunPrivate::runExecutable(const QString &_exec)
         }
     } else {
         KService::Ptr service = KService::serviceByStorageId(_exec);
-        if (service && q->run(*service, urls, m_window, false, QString(), m_asn)) {
+        if (service && q->runService(*service, urls, m_window, false, QString(), m_asn)) {
             m_bFinished = true;
             startTimer();
             return true;
@@ -987,17 +1010,17 @@ bool KRun::KRunPrivate::isPromptNeeded()
     const QMimeType mime = db.mimeTypeForUrl(m_strURL);
 
     const bool isFileExecutable = (isExecutableFile(m_strURL, mime.name()) ||
-                                   mime.inherits("application/x-desktop"));
-    const bool isTextFile = mime.inherits("text/plain");
+                                   mime.inherits(QStringLiteral("application/x-desktop")));
+    const bool isTextFile = mime.inherits(QStringLiteral("text/plain"));
 
     if (isFileExecutable && isTextFile) {
-        KConfigGroup cfgGroup(KSharedConfig::openConfig("kiorc"), "Executable scripts");
+        KConfigGroup cfgGroup(KSharedConfig::openConfig(QStringLiteral("kiorc")), "Executable scripts");
         const QString value = cfgGroup.readEntry("behaviourOnLaunch", "alwaysAsk");
 
-        if (value == "alwaysAsk") {
+        if (value == QLatin1String("alwaysAsk")) {
             return true;
         } else {
-            q->setRunExecutables(value == "execute");
+            q->setRunExecutables(value == QLatin1String("execute"));
         }
     }
 
@@ -1016,7 +1039,7 @@ void KRun::KRunPrivate::onDialogFinished(int result, bool isDontAskAgainSet)
 
     if (isDontAskAgainSet) {
         QString output = result == ExecutableFileOpenDialog::OpenFile ? "open" : "execute";
-        KConfigGroup cfgGroup(KSharedConfig::openConfig("kiorc"), "Executable scripts");
+        KConfigGroup cfgGroup(KSharedConfig::openConfig(QStringLiteral("kiorc")), "Executable scripts");
         cfgGroup.writeEntry("behaviourOnLaunch", output);
     }
     startTimer();
@@ -1090,7 +1113,7 @@ void KRun::slotTimeout()
             return;
         } else if (d->m_bIsDirectory) {
             d->m_bIsDirectory = false;
-            mimeTypeDetermined("inode/directory");
+            mimeTypeDetermined(QStringLiteral("inode/directory"));
             return;
         }
     }
@@ -1227,7 +1250,7 @@ void KRun::foundMimeType(const QString &type)
         if (serv && serv->hasMimeType(type)) {
             QList<QUrl> lst;
             lst.append(d->m_strURL);
-            if (KRun::run(*serv, lst, d->m_window, false, QString(), d->m_asn)) {
+            if (KRun::runService(*serv, lst, d->m_window, false, QString(), d->m_asn)) {
                 setFinished(true);
                 return;
             }
@@ -1242,7 +1265,7 @@ void KRun::foundMimeType(const QString &type)
     QMimeType mime = db.mimeTypeForName(type);
     if (!mime.isValid()) {
         qWarning() << "Unknown mimetype " << type;
-    } else if (mime.inherits("application/x-desktop") && !d->m_localPath.isEmpty()) {
+    } else if (mime.inherits(QStringLiteral("application/x-desktop")) && !d->m_localPath.isEmpty()) {
         d->m_strURL = QUrl::fromLocalFile(d->m_localPath);
     }
 
@@ -1343,10 +1366,10 @@ QString KRun::suggestedFileName() const
 
 bool KRun::isExecutable(const QString &serviceType)
 {
-    return (serviceType == "application/x-desktop" ||
-            serviceType == "application/x-executable" ||
-            serviceType == "application/x-ms-dos-executable" ||
-            serviceType == "application/x-shellscript");
+    return (serviceType == QLatin1String("application/x-desktop") ||
+            serviceType == QLatin1String("application/x-executable") ||
+            serviceType == QLatin1String("application/x-ms-dos-executable") ||
+            serviceType == QLatin1String("application/x-shellscript"));
 }
 
 void KRun::setUrl(const QUrl &url)
@@ -1446,40 +1469,28 @@ bool KRun::isLocalFile() const
 
 /****************/
 
-qint64 KProcessRunner::run(const QString &command, const QString &executable, const KStartupInfoId &id, const QString &workingDirectory)
+qint64 KProcessRunner::run(KProcess *p, const QString &executable, const KStartupInfoId &id)
 {
-    return (new KProcessRunner(command, executable, id, workingDirectory))->pid();
+    return (new KProcessRunner(p, executable, id))->pid();
 }
 
-KProcessRunner::KProcessRunner(const QString &command, const QString &executable, const KStartupInfoId &id, const QString &workingDirectory) :
+KProcessRunner::KProcessRunner(KProcess *p, const QString &executable, const KStartupInfoId &id) :
     id(id)
 {
     m_pid = 0;
-    process = new QProcess;
+    process = p;
     m_executable = executable;
     connect(process, SIGNAL(finished(int,QProcess::ExitStatus)),
             this, SLOT(slotProcessExited(int,QProcess::ExitStatus)));
 
-    if (!workingDirectory.isEmpty()) {
-        process->setWorkingDirectory(workingDirectory);
-    }
-
-    process->start(command);
+    process->start();
     if (!process->waitForStarted()) {
         //qDebug() << "wait for started failed, exitCode=" << process->exitCode()
         //         << "exitStatus=" << process->exitStatus();
         // Note that exitCode is 255 here (the first time), and 0 later on (bug?).
         slotProcessExited(255, process->exitStatus());
     } else {
-#if QT_VERSION >= QT_VERSION_CHECK(5, 3, 0)
         m_pid = process->processId();
-#pragma message("TODO: remove the other cases when we depend on Qt 5.3")
-#elif defined(Q_OS_WIN)
-        // QProcess::pid() returns a pointer to a struct on windows
-        m_pid = process->pid()->dwProcessId;
-#else
-        m_pid = process->pid();
-#endif
     }
 }
 
