@@ -60,7 +60,7 @@ private:
             : KDragWidgetDecoratorBase(button), m_button(button) {}
 
     protected:
-        virtual QDrag *dragObject()
+        QDrag *dragObject() Q_DECL_OVERRIDE
         {
             if (m_button->m_urls.isEmpty()) {
                 return 0;
@@ -225,13 +225,13 @@ public:
             int sep = it->indexOf('|');
             QString globs = it->left(sep);
             QString desc  = it->mid(sep + 1);
-            *it = QString("%1 (%2)").arg(desc).arg(globs);
+            *it = QStringLiteral("%1 (%2)").arg(desc).arg(globs);
         }
 
         return qFilters;
     }
 
-    //QFileDialog::getExistingDirectoryDirectoryUrl only works with local for any version before
+    //QFileDialog::getExistingDirectoryUrl only works with local for any version before
     //5.4, so we have to check our own
     QUrl getDirFromFileDialog(const QUrl &openUrl) const
     {
@@ -316,7 +316,7 @@ void KUrlRequester::KUrlRequesterPrivate::init()
     topLayout->addWidget(widget);
 
     myButton = new KUrlDragPushButton(m_parent);
-    myButton->setIcon(QIcon::fromTheme("document-open"));
+    myButton->setIcon(QIcon::fromTheme(QStringLiteral("document-open")));
     int buttonSize = myButton->sizeHint().expandedTo(widget->sizeHint()).height();
     myButton->setFixedSize(buttonSize, buttonSize);
     myButton->setToolTip(i18n("Open file dialog"));
@@ -414,7 +414,7 @@ void KUrlRequester::KUrlRequesterPrivate::_k_slotOpenDialog()
 
         QUrl newUrl;
         if (fileDialogMode & KFile::LocalOnly) {
-            newUrl = QUrl::fromLocalFile(QFileDialog::getExistingDirectory(m_parent, QString(), openUrl.toString(), QFileDialog::ShowDirsOnly));
+            newUrl = QFileDialog::getExistingDirectoryUrl(m_parent, QString(), openUrl, QFileDialog::ShowDirsOnly, QStringList() << QStringLiteral("file"));
         } else {
             newUrl = getDirFromFileDialog(openUrl);
         }
